@@ -37,7 +37,7 @@
 #define expect(expected,got) expect_(__LINE__, expected, got)
 static inline void expect_(unsigned line, DWORD expected, DWORD got)
 {
-    ok_(__FILE__, line)(expected == got, "Expected %d, got %d\n", expected, got);
+    ok_(__FILE__, line)(expected == got, "Expected %ld, got %ld\n", expected, got);
 }
 
 #define expect_str(expected, got)\
@@ -56,14 +56,14 @@ static void CheckSize(HWND hwnd, INT width, INT height, const char *msg, int lin
 
     SendMessageA(hwnd, TCM_GETITEMRECT, 0, (LPARAM)&r);
     if (width >= 0 && height < 0)
-        ok_(__FILE__,line) (width == r.right - r.left, "%s: Expected width [%d] got [%d]\n",
+        ok_(__FILE__,line) (width == r.right - r.left, "%s: Expected width [%d] got [%ld]\n",
             msg, width, r.right - r.left);
     else if (height >= 0 && width < 0)
-        ok_(__FILE__,line) (height == r.bottom - r.top,  "%s: Expected height [%d] got [%d]\n",
+        ok_(__FILE__,line) (height == r.bottom - r.top,  "%s: Expected height [%d] got [%ld]\n",
             msg, height, r.bottom - r.top);
     else
         ok_(__FILE__,line) ((width  == r.right  - r.left) && (height == r.bottom - r.top ),
-	    "%s: Expected [%d,%d] got [%d,%d]\n", msg, width, height,
+	    "%s: Expected [%d,%d] got [%ld,%ld]\n", msg, width, height,
             r.right - r.left, r.bottom - r.top);
 }
 
@@ -85,9 +85,54 @@ static HFONT hFont;
 static DRAWITEMSTRUCT g_drawitem;
 static HWND parent_wnd;
 static LRESULT tcn_selchanging_result;
+static DWORD winevent_hook_thread_id;
 
 static struct msg_sequence *sequences[NUM_MSG_SEQUENCES];
 
+static void CALLBACK msg_winevent_proc(HWINEVENTHOOK hevent,
+				    DWORD event,
+				    HWND hwnd,
+				    LONG object_id,
+				    LONG child_id,
+				    DWORD thread_id,
+				    DWORD event_time)
+{
+    struct message msg = {0};
+    char class_name[256];
+
+    ok(thread_id == winevent_hook_thread_id, "we didn't ask for events from other threads\n");
+
+    /* ignore window and other system events */
+    if (object_id != OBJID_CLIENT) return;
+
+    /* ignore events not from a tab control */
+    if (!GetClassNameA(hwnd, class_name, ARRAY_SIZE(class_name)) ||
+        strcmp(class_name, WC_TABCONTROLA) != 0)
+        return;
+
+    msg.message = event;
+    msg.flags = winevent_hook|wparam|lparam;
+    msg.wParam = object_id;
+    msg.lParam = child_id;
+    add_message(sequences, TAB_SEQ_INDEX, &msg);
+}
+
+static void init_winevent_hook(void) {
+    hwineventhook = SetWinEventHook(EVENT_MIN, EVENT_MAX, GetModuleHandleA(0), msg_winevent_proc,
+        0, GetCurrentThreadId(), WINEVENT_INCONTEXT);
+    winevent_hook_thread_id = GetCurrentThreadId();
+    if (!hwineventhook)
+        win_skip( "no win event hook support\n" );
+}
+
+static void uninit_winevent_hook(void) {
+    if (!hwineventhook)
+        return;
+
+    UnhookWinEvent(hwineventhook);
+    hwineventhook = 0;
+}
+
 static const struct message empty_sequence[] = {
     { 0 }
 };
@@ -99,10 +144,28 @@ static const struct message get_row_count_seq[] = {
 
 static const struct message getset_cur_focus_seq[] = {
     { TCM_SETCURFOCUS, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 5 },
+    { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
+    { TCM_SETCURFOCUS, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 0 },
+    { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
+    { TCM_SETCURSEL, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 2 },
+    { TCM_SETCURFOCUS, sent|lparam, 0 },
+    { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
+    { 0 }
+};
+
+static const struct message getset_cur_focus_buttons_seq[] = {
+    { TCM_SETCURFOCUS, sent|lparam, 0 },
+    { TCM_GETITEMCOUNT, sent|defwinproc|optional },
+    { TCM_GETITEMCOUNT, sent|defwinproc|optional },
+    { EVENT_OBJECT_FOCUS, winevent_hook|wparam|lparam, OBJID_CLIENT, 5 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_SETCURFOCUS, sent|lparam, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_SETCURSEL, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 2 },
     { TCM_SETCURFOCUS, sent|lparam, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { 0 }
@@ -110,11 +173,14 @@ static const struct message getset_cur_focus_seq[] = {
 
 static const struct message getset_cur_sel_seq[] = {
     { TCM_SETCURSEL, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 5 },
     { TCM_GETCURSEL, sent|wparam|lparam, 0, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_SETCURSEL, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 0 },
     { TCM_GETCURSEL, sent|wparam|lparam, 0, 0 },
     { TCM_SETCURSEL, sent|lparam, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 2 },
     { TCM_SETCURSEL, sent|lparam, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { 0 }
@@ -166,6 +232,7 @@ static const struct message insert_focus_seq[] = {
     { TCM_GETITEMCOUNT, sent|wparam|lparam, 0, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_INSERTITEMA, sent|wparam, 1 },
+    { EVENT_OBJECT_CREATE, winevent_hook|wparam|lparam, OBJID_CLIENT, 1 },
     { WM_NOTIFYFORMAT, sent|defwinproc|optional },
     { WM_QUERYUISTATE, sent|defwinproc|optional },
     { WM_PARENTNOTIFY, sent|defwinproc|optional },
@@ -175,11 +242,14 @@ static const struct message insert_focus_seq[] = {
     { WM_NOTIFYFORMAT, sent|defwinproc|optional },
     { WM_QUERYUISTATE, sent|defwinproc|optional, },
     { WM_PARENTNOTIFY, sent|defwinproc|optional },
+    { EVENT_OBJECT_CREATE, winevent_hook|wparam|lparam, OBJID_CLIENT, 2 },
     { TCM_GETITEMCOUNT, sent|wparam|lparam, 0, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_SETCURFOCUS, sent|wparam|lparam, -1, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_INSERTITEMA, sent|wparam, 3 },
+    { EVENT_OBJECT_CREATE, winevent_hook|wparam|lparam, OBJID_CLIENT, 3 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { 0 }
 };
@@ -188,11 +258,14 @@ static const struct message delete_focus_seq[] = {
     { TCM_GETITEMCOUNT, sent|wparam|lparam, 0, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_DELETEITEM, sent|wparam|lparam, 1, 0 },
+    { EVENT_OBJECT_DESTROY, winevent_hook|wparam|lparam, OBJID_CLIENT, 2 },
     { TCM_GETITEMCOUNT, sent|wparam|lparam, 0, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_SETCURFOCUS, sent|wparam|lparam, -1, 0 },
+    { EVENT_OBJECT_SELECTION, winevent_hook|wparam|lparam, OBJID_CLIENT, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { TCM_DELETEITEM, sent|wparam|lparam, 0, 0 },
+    { EVENT_OBJECT_DESTROY, winevent_hook|wparam|lparam, OBJID_CLIENT, 1 },
     { TCM_GETITEMCOUNT, sent|wparam|lparam, 0, 0 },
     { TCM_GETCURFOCUS, sent|wparam|lparam, 0, 0 },
     { 0 }
@@ -347,7 +420,8 @@ static LRESULT WINAPI tab_subclass_proc(HWND hwnd, UINT message, WPARAM wParam,
         message != WM_NCHITTEST &&
         message != WM_GETTEXT &&
         message != WM_GETICON &&
-        message != WM_DEVICECHANGE)
+        message != WM_DEVICECHANGE &&
+        message != WM_GETOBJECT)
     {
         msg.message = message;
         msg.flags = sent|wparam|lparam;
@@ -461,7 +535,7 @@ static void test_tab(INT nMinTabWidth)
     dpi = GetDeviceCaps(hdc, LOGPIXELSX);
     hOldFont = SelectObject(hdc, (HFONT)SendMessageA(hwTab, WM_GETFONT, 0, 0));
     GetTextExtentPoint32A(hdc, "Tab 1", strlen("Tab 1"), &size);
-    trace("Tab1 text size: size.cx=%d size.cy=%d\n", size.cx, size.cy);
+    trace("Tab1 text size: size.cx=%ld size.cy=%ld\n", size.cx, size.cy);
     SelectObject(hdc, hOldFont);
     ReleaseDC(hwTab, hdc);
 
@@ -532,7 +606,7 @@ static void test_tab(INT nMinTabWidth)
     exp = max(size.cx +TAB_PADDING_X*2, (nMinTabWidth < 0) ? DEFAULT_MIN_TAB_WIDTH : nMinTabWidth);
     SendMessageA( hwTab, TCM_GETITEMRECT, 0, (LPARAM)&rTab );
     ok( rTab.right  - rTab.left == exp || broken(rTab.right  - rTab.left == DEFAULT_MIN_TAB_WIDTH),
-        "no icon, default width: Expected width [%d] got [%d]\n", exp, rTab.right - rTab.left );
+        "no icon, default width: Expected width [%d] got [%ld]\n", exp, rTab.right - rTab.left );
 
     for (i=0; i<8; i++)
     {
@@ -560,7 +634,7 @@ static void test_tab(INT nMinTabWidth)
     exp = (nMinTabWidth < 0) ? DEFAULT_MIN_TAB_WIDTH : nMinTabWidth;
     SendMessageA( hwTab, TCM_GETITEMRECT, 0, (LPARAM)&rTab );
     ok( rTab.right  - rTab.left == exp || broken(rTab.right  - rTab.left == DEFAULT_MIN_TAB_WIDTH),
-        "no icon, default width: Expected width [%d] got [%d]\n", exp, rTab.right - rTab.left );
+        "no icon, default width: Expected width [%d] got [%ld]\n", exp, rTab.right - rTab.left );
 
     for (i=0; i<8; i++)
     {
@@ -667,7 +741,7 @@ static void test_curfocus(void)
     todo_wine
     ok(ret == nTabs - 1, "Unexpected focus index %d.\n", ret);
 
-    ok_sequence(sequences, TAB_SEQ_INDEX, getset_cur_focus_seq, "TCS_BUTTONS: set focused tab sequence", FALSE);
+    ok_sequence(sequences, TAB_SEQ_INDEX, getset_cur_focus_buttons_seq, "TCS_BUTTONS: set focused tab sequence", FALSE);
     ok_sequence(sequences, PARENT_SEQ_INDEX, setfocus_parent_seq, "TCS_BUTTONS: set focused tab parent sequence", TRUE);
 
     DestroyWindow(hTab);
@@ -830,14 +904,14 @@ static void test_getset_item(void)
     ok(GetParent(hTab) == NULL, "got %p, expected null parent\n", GetParent(hTab));
 
     ret = SendMessageA(hTab, TCM_SETITEMEXTRA, sizeof(LPARAM)-1, 0);
-    ok(ret == TRUE, "got %d\n", ret);
+    ok(ret == TRUE, "got %ld\n", ret);
 
     /* set some item data */
     tcItem.lParam = ~0;
     tcItem.mask = TCIF_PARAM;
 
     ret = SendMessageA(hTab, TCM_INSERTITEMA, 0, (LPARAM)&tcItem);
-    ok(ret == 0, "got %d\n", ret);
+    ok(ret == 0, "got %ld\n", ret);
 
     /* all sizeof(LPARAM) returned anyway when using sizeof(LPARAM)-1 size */
     memset(&lparam, 0xaa, sizeof(lparam));
@@ -847,7 +921,7 @@ static void test_getset_item(void)
     expect(TRUE, ret);
     /* everything higher specified size is preserved */
     memset(&lparam, 0xff, sizeof(lparam)-1);
-    ok(tcItem.lParam == lparam, "Expected 0x%lx, got 0x%lx\n", lparam, tcItem.lParam);
+    ok(tcItem.lParam == lparam, "Expected 0x%Ix, got 0x%Ix\n", lparam, tcItem.lParam);
 
     DestroyWindow(hTab);
 
@@ -862,7 +936,7 @@ static void test_getset_item(void)
     tcItem.mask = TCIF_PARAM;
     ret = SendMessageA(hTab, TCM_GETITEMA, 5, (LPARAM)&tcItem);
     expect(FALSE, ret);
-    ok(tcItem.lParam == 0, "Expected zero lParam, got %lu\n", tcItem.lParam);
+    ok(tcItem.lParam == 0, "Expected zero lParam, got %Iu\n", tcItem.lParam);
 
     memset(&tcItem, 0xcc, sizeof(tcItem));
     tcItem.mask = TCIF_IMAGE;
@@ -899,13 +973,13 @@ static void test_getset_item(void)
     tcItem.mask = TCIF_PARAM;
     ret = SendMessageA(hTab, TCM_GETITEMA, -1, (LPARAM)&tcItem);
     expect(FALSE, ret);
-    ok(tcItem.lParam == 0, "Expected zero lParam, got %lu\n", tcItem.lParam);
+    ok(tcItem.lParam == 0, "Expected zero lParam, got %Iu\n", tcItem.lParam);
 
     memset(&tcItem, 0xcc, sizeof(tcItem));
     tcItem.mask = TCIF_PARAM;
     ret = SendMessageA(hTab, TCM_GETITEMA, -2, (LPARAM)&tcItem);
     expect(FALSE, ret);
-    ok(tcItem.lParam == 0, "Expected zero lParam, got %lu\n", tcItem.lParam);
+    ok(tcItem.lParam == 0, "Expected zero lParam, got %Iu\n", tcItem.lParam);
 
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
 
@@ -1227,10 +1301,10 @@ static void test_TCM_SETITEMEXTRA(void)
     }
 
     ret = SendMessageA(hTab, TCM_SETITEMEXTRA, -1, 0);
-    ok(ret == FALSE, "got %d\n", ret);
+    ok(ret == FALSE, "got %ld\n", ret);
 
     ret = SendMessageA(hTab, TCM_SETITEMEXTRA, 2, 0);
-    ok(ret == TRUE, "got %d\n", ret);
+    ok(ret == TRUE, "got %ld\n", ret);
     DestroyWindow(hTab);
 
     /* it's not possible to change extra data size for control with tabs */
@@ -1238,7 +1312,7 @@ static void test_TCM_SETITEMEXTRA(void)
     ok(hTab != NULL, "Failed to create tab control\n");
 
     ret = SendMessageA(hTab, TCM_SETITEMEXTRA, 2, 0);
-    ok(ret == FALSE, "got %d\n", ret);
+    ok(ret == FALSE, "got %ld\n", ret);
     DestroyWindow(hTab);
 }
 
@@ -1270,7 +1344,7 @@ static void test_TCS_OWNERDRAWFIXED(void)
 
     itemdata = 0;
     memset(&itemdata, 0xde, 4);
-    ok(g_drawitem.itemData == itemdata, "got 0x%lx, expected 0x%lx\n", g_drawitem.itemData, itemdata);
+    ok(g_drawitem.itemData == itemdata, "got 0x%Ix, expected 0x%Ix\n", g_drawitem.itemData, itemdata);
 
     DestroyWindow(hTab);
 
@@ -1301,7 +1375,7 @@ static void test_TCS_OWNERDRAWFIXED(void)
     RedrawWindow(hTab, NULL, 0, RDW_UPDATENOW);
 
     memset(&itemdata, 0xde, sizeof(ULONG_PTR));
-    ok(*(ULONG_PTR*)g_drawitem.itemData == itemdata, "got 0x%lx, expected 0x%lx\n", g_drawitem.itemData, itemdata);
+    ok(*(ULONG_PTR*)g_drawitem.itemData == itemdata, "got 0x%Ix, expected 0x%Ix\n", g_drawitem.itemData, itemdata);
 
     DestroyWindow(hTab);
 
@@ -1334,7 +1408,7 @@ static void test_TCS_OWNERDRAWFIXED(void)
     memset(&itemdata, 0xde, 4);
     memset(&itemdata2, 0xde, sizeof(LPARAM)-1);
     ok(g_drawitem.itemData == itemdata || broken(g_drawitem.itemData == itemdata2) /* win98 */,
-        "got 0x%lx, expected 0x%lx\n", g_drawitem.itemData, itemdata);
+        "got 0x%Ix, expected 0x%Ix\n", g_drawitem.itemData, itemdata);
 
     DestroyWindow(hTab);
 }
@@ -1378,7 +1452,7 @@ static void test_create(void)
         hTab = CreateWindowA(WC_TABCONTROLA, "TestTab", ptr->style,
             10, 10, 300, 100, parent_wnd, NULL, NULL, 0);
         style = GetWindowLongA(hTab, GWL_STYLE);
-        ok(style == ptr->act_style, "expected style 0x%08x, got style 0x%08x\n", ptr->act_style, style);
+        ok(style == ptr->act_style, "expected style 0x%08lx, got style 0x%08lx\n", ptr->act_style, style);
 
         DestroyWindow(hTab);
         ptr++;
@@ -1501,6 +1575,8 @@ START_TEST(tab)
 
     init_msg_sequences(sequences, NUM_MSG_SEQUENCES);
 
+    init_winevent_hook();
+
     parent_wnd = createParentWindow();
     ok(parent_wnd != NULL, "Failed to create parent window!\n");
 
@@ -1523,5 +1599,7 @@ START_TEST(tab)
     test_TCN_SELCHANGING();
     test_TCM_GETROWCOUNT();
 
+    uninit_winevent_hook();
+
     DestroyWindow(parent_wnd);
 }
