@@ -43,6 +43,32 @@ static char mdbpath[MAX_PATH];
 
 static const VARTYPE intptr_vartype = (sizeof(void *) == 8 ? VT_I8 : VT_I4);
 
+static void free_dbpropset(ULONG count, DBPROPSET *propset)
+{
+    ULONG i, j;
+
+    for (i = 0; i < count; i++)
+    {
+        for (j = 0; j < propset[i].cProperties; j++)
+            VariantClear(&propset[i].rgProperties[j].vValue);
+        CoTaskMemFree(propset[i].rgProperties);
+    }
+    CoTaskMemFree(propset);
+}
+
+static void free_dbpropinfoset(ULONG count, DBPROPINFOSET *propinfoset)
+{
+    ULONG i, j;
+
+    for (i = 0; i < count; i++)
+    {
+        for (j = 0; j < propinfoset[i].cPropertyInfos; j++)
+            VariantClear(&propinfoset[i].rgPropertyInfos[j].vValues);
+        CoTaskMemFree(propinfoset[i].rgPropertyInfos);
+    }
+    CoTaskMemFree(propinfoset);
+}
+
 static void test_msdasql(void)
 {
     HRESULT hr;
@@ -125,12 +151,10 @@ static void test_Properties(void)
             propidlist.rgPropertyIDs[i] = propinfoset->rgPropertyInfos[i].dwPropertyID;
         }
 
-        for (i = 0; i < propinfoset->cPropertyInfos; i++)
-            VariantClear(&propinfoset->rgPropertyInfos[i].vValues);
-
-        CoTaskMemFree(propinfoset->rgPropertyInfos);
-        CoTaskMemFree(propinfoset);
+        free_dbpropinfoset(infocount, propinfoset);
+        CoTaskMemFree(desc);
 
+        /* Test specifying all supported properties */
         hr = IDBProperties_GetProperties(props, 1, &propidlist, &propcnt, &propset);
         ok(hr == S_OK, "got 0x%08lx\n", hr);
         ok(propidlist.cPropertyIDs == 14, "got %lu\n", propidlist.cPropertyIDs);
@@ -161,8 +185,41 @@ static void test_Properties(void)
             ok(V_VT(&propset->rgProperties[i].vValue) == vartype, "%lu wrong type %d\n", i, V_VT(&propset->rgProperties[i].vValue));
         }
 
+        for (i = 0; i < propset->cProperties; i++)
+            ok(propset->rgProperties[i].dwPropertyID == properties[i],
+               "%ld %ld, got %ld\n", i, properties[i], propset->rgProperties[i].dwPropertyID);
+
+        free_dbpropset(propcnt, propset);
+
+        /* Test specifying only one supported properties */
+        propidlist.cPropertyIDs = 1;
+        hr = IDBProperties_GetProperties(props, 1, &propidlist, &propcnt, &propset);
+        ok(hr == S_OK, "got 0x%08lx\n", hr);
+        ok(propset->cProperties == 1, "expected 1, got %lu\n", propset->cProperties);
+        free_dbpropset(propcnt, propset);
+
+        /* Test when cPropertyIDSets is zero, all initialization properties should be returned */
+        hr = IDBProperties_GetProperties(props, 0, &propidlist, &propcnt, &propset);
+        ok(hr == S_OK, "got 0x%08lx\n", hr);
+        ok(propset->cProperties == ARRAY_SIZE(properties), "got %lu\n", propset->cProperties);
+        for (i = 0; i < propset->cProperties; i++)
+             ok(propset->rgProperties[i].dwPropertyID == properties[i],
+                "%ld %ld, got %ld\n", i, properties[i], propset->rgProperties[i].dwPropertyID);
+        free_dbpropset(propcnt, propset);
+
+        /* Test when propidlist.cPropertyIDs is zero, all initialization properties should be returned */
         CoTaskMemFree(propidlist.rgPropertyIDs);
-        CoTaskMemFree(propset);
+        propidlist.guidPropertySet = DBPROPSET_DBINIT;
+        propidlist.cPropertyIDs = 0;
+        propidlist.rgPropertyIDs = NULL;
+
+        hr = IDBProperties_GetProperties(props, 1, &propidlist, &propcnt, &propset);
+        ok(hr == S_OK, "got 0x%08lx\n", hr);
+        ok(propset->cProperties == ARRAY_SIZE(properties), "got %lu\n", propset->cProperties);
+        for (i = 0; i < propset->cProperties; i++)
+            ok(propset->rgProperties[i].dwPropertyID == properties[i],
+               "%ld %ld, got %ld\n", i, properties[i], propset->rgProperties[i].dwPropertyID);
+        free_dbpropset(propcnt, propset);
     }
 
     propid = DBPROP_MULTIPLERESULTS;
@@ -179,7 +236,7 @@ static void test_Properties(void)
     ok(propset->rgProperties[0].dwPropertyID == DBPROP_MULTIPLERESULTS, "got %ld\n", propset->rgProperties[0].dwPropertyID);
     ok(propset->rgProperties[0].dwStatus == DBPROPSTATUS_NOTSUPPORTED, "got %ld\n", propset->rgProperties[0].dwStatus);
 
-    CoTaskMemFree(propset);
+    free_dbpropset(propcnt, propset);
 
     IDBProperties_Release(props);
 }
@@ -324,7 +381,7 @@ static void test_command_properties(ICommandProperties *props)
         }
     }
 
-    CoTaskMemFree(propset);
+    free_dbpropset(count, propset);
 }
 
 static void test_command_interfaces(IUnknown *cmd)
@@ -496,6 +553,42 @@ static void test_rowset_interfaces(IRowset *rowset, ICommandText *commandtext)
     ok(hr == E_NOINTERFACE, "got 0x%08lx\n", hr);
 }
 
+static void test_rowset_info(IRowset *rowset)
+{
+    IRowsetInfo *info;
+    HRESULT hr;
+    ULONG propcnt;
+    DBPROPIDSET propidset;
+    DBPROPSET *propset;
+    int i;
+    DWORD row_props[] = {
+            DBPROP_CANSCROLLBACKWARDS, DBPROP_IRowsetUpdate, DBPROP_IRowsetResynch,
+            DBPROP_IConnectionPointContainer, DBPROP_BOOKMARKSKIPPED, DBPROP_REMOVEDELETED,
+            DBPROP_IConvertType, DBPROP_NOTIFICATIONGRANULARITY, DBPROP_IMultipleResults, DBPROP_ACCESSORDER,
+            DBPROP_BOOKMARKINFO, DBPROP_UNIQUEROWS
+    };
+
+    hr = IRowset_QueryInterface(rowset, &IID_IRowsetInfo, (void**)&info);
+    ok(hr == S_OK, "got 0x%08lx\n", hr);
+
+    propidset.rgPropertyIDs = row_props;
+    propidset.cPropertyIDs = ARRAY_SIZE(row_props);
+    propidset.guidPropertySet = DBPROPSET_ROWSET;
+
+    hr = IRowsetInfo_GetProperties(info, 1, &propidset, &propcnt, &propset);
+    ok(hr == S_OK, "got 0x%08lx\n", hr);
+    ok(propset->cProperties == ARRAY_SIZE(row_props), "got %lu\n", propset->cProperties);
+
+    for(i=0; i < ARRAY_SIZE(row_props); i++)
+    {
+        ok(propset->rgProperties[i].dwPropertyID == row_props[i], "expected 0x%08lx got 0x%08lx\n",
+                propset->rgProperties[i].dwPropertyID, row_props[i]);
+    }
+
+    free_dbpropset(propcnt, propset);
+    IRowsetInfo_Release(info);
+}
+
 static void test_command_rowset(IUnknown *cmd)
 {
     ICommandText *command_text;
@@ -517,7 +610,7 @@ static void test_command_rowset(IUnknown *cmd)
     hr = ICommandPrepare_Prepare(commandprepare, 1);
     ok(hr == DB_E_NOCOMMAND, "got 0x%08lx\n", hr);
 
-    hr = ICommandText_SetCommandText(command_text, &DBGUID_DEFAULT, L"CREATE TABLE testing (col1 INT, col2 SHORT)");
+    hr = ICommandText_SetCommandText(command_text, &DBGUID_DEFAULT, L"CREATE TABLE testing (col1 INT, col2 VARCHAR(20) NOT NULL, col3 FLOAT)");
     ok(hr == S_OK, "got 0x%08lx\n", hr);
 
     hr = ICommandPrepare_Prepare(commandprepare, 1);
@@ -532,7 +625,14 @@ static void test_command_rowset(IUnknown *cmd)
     if (unk)
         IUnknown_Release(unk);
 
-    hr = ICommandText_SetCommandText(command_text, &DBGUID_DEFAULT, L"insert into testing values(1, 0)");
+    /* Ensure all rows are deleted - Interactive Test */
+    hr = ICommandText_SetCommandText(command_text, &DBGUID_DEFAULT, L"delete from testing");
+    ok(hr == S_OK, "got 0x%08lx\n", hr);
+
+    hr = ICommandText_Execute(command_text, NULL, &IID_NULL, NULL, NULL, NULL);
+    ok(hr == S_OK, "got 0x%08lx\n", hr);
+
+    hr = ICommandText_SetCommandText(command_text, &DBGUID_DEFAULT, L"insert into testing values(1, 'red', 1.0)");
     ok(hr == S_OK, "got 0x%08lx\n", hr);
 
     affected = 9999;
@@ -550,13 +650,45 @@ static void test_command_rowset(IUnknown *cmd)
     ok(unk != NULL, "Unexpected value\n");
     if (hr == S_OK)
     {
-        ok(affected == -1, "got %Id\n", affected);
+        const DWORD flag1 = DBCOLUMNFLAGS_ISFIXEDLENGTH | DBCOLUMNFLAGS_ISNULLABLE | DBCOLUMNFLAGS_MAYBENULL | DBCOLUMNFLAGS_WRITE;
+        const DWORD flag2 = DBCOLUMNFLAGS_ISNULLABLE | DBCOLUMNFLAGS_MAYBENULL | DBCOLUMNFLAGS_WRITE;
+        IColumnsInfo *colinfo;
+        DBORDINAL columns;
+        DBCOLUMNINFO *dbcolinfo;
+        OLECHAR *stringsbuffer;
+
+        ok(affected == -1 || affected == 1, "got %Iu\n", affected);
 
         hr = IUnknown_QueryInterface(unk, &IID_IRowset, (void**)&rowset);
         ok(hr == S_OK, "got 0x%08lx\n", hr);
 
         test_rowset_interfaces(rowset, command_text);
 
+        hr = IRowset_QueryInterface(rowset, &IID_IColumnsInfo, (void**)&colinfo);
+        ok(hr == S_OK, "got 0x%08lx\n", hr);
+
+        columns = 0;
+        hr = IColumnsInfo_GetColumnInfo(colinfo, &columns, &dbcolinfo, &stringsbuffer);
+        ok(hr == S_OK, "got 0x%08lx\n", hr);
+        ok(columns == 3, "got %Iu\n", columns);
+
+        ok(dbcolinfo[0].dwFlags == flag1, "got 0x%08lx\n", dbcolinfo[0].dwFlags);
+        ok(dbcolinfo[0].wType == DBTYPE_I4, "got 0x%08x\n", dbcolinfo[0].wType);
+
+        todo_wine ok(dbcolinfo[1].dwFlags == flag2, "got 0x%08lx\n", dbcolinfo[1].dwFlags);
+        ok(dbcolinfo[1].wType == DBTYPE_WSTR /* Unicode MySQL Driver */ ||
+           dbcolinfo[1].wType == DBTYPE_STR  /* ASCII MySQL Driver */, "got 0x%08x\n", dbcolinfo[1].wType);
+
+        ok(dbcolinfo[2].dwFlags == flag1, "got 0x%08lx\n", dbcolinfo[2].dwFlags);
+        ok(dbcolinfo[2].wType == DBTYPE_R4 /* MySQL */ ||
+           dbcolinfo[2].wType == DBTYPE_R8 /* Access */, "got 0x%08x\n", dbcolinfo[2].wType);
+
+        CoTaskMemFree(dbcolinfo);
+        CoTaskMemFree(stringsbuffer);
+        IColumnsInfo_Release(colinfo);
+
+        test_rowset_info(rowset);
+
         IRowset_Release(rowset);
         IUnknown_Release(unk);
     }
