@@ -69,7 +69,10 @@ static nsIClipboardCommands *get_clipboard_commands(HTMLDocumentNode *doc)
     nsIDocShell *doc_shell;
     nsresult nsres;
 
-    nsres = get_nsinterface((nsISupports*)doc->outer_window->nswindow, &IID_nsIDocShell, (void**)&doc_shell);
+    if(!doc->window)
+        return NULL;
+
+    nsres = get_nsinterface((nsISupports*)doc->window->dom_window, &IID_nsIDocShell, (void**)&doc_shell);
     if(NS_FAILED(nsres)) {
         ERR("Could not get nsIDocShell interface\n");
         return NULL;
@@ -438,7 +441,7 @@ static void refresh_proc(task_t *_task)
         IOleCommandTarget_Exec(window->browser->doc->client_cmdtrg, &CGID_ShellDocView, 37, 0, &var, NULL);
     }
 
-    load_uri(task->window, task->window->uri, BINDING_REFRESH|BINDING_NOFRAG);
+    load_uri(window, window->uri, BINDING_REFRESH|BINDING_NOFRAG);
 }
 
 static void refresh_destr(task_t *_task)
@@ -446,13 +449,25 @@ static void refresh_destr(task_t *_task)
     refresh_task_t *task = (refresh_task_t*)_task;
 
     IHTMLWindow2_Release(&task->window->base.IHTMLWindow2_iface);
-    free(task);
+}
+
+HRESULT reload_page(HTMLOuterWindow *window)
+{
+    refresh_task_t *task;
+
+    task = malloc(sizeof(*task));
+    if(!task)
+        return E_OUTOFMEMORY;
+
+    IHTMLWindow2_AddRef(&window->base.IHTMLWindow2_iface);
+    task->window = window;
+
+    return push_task(&task->header, refresh_proc, refresh_destr, window->task_magic);
 }
 
 static HRESULT exec_refresh(HTMLDocumentNode *doc, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
 {
     HTMLDocumentObj *doc_obj;
-    refresh_task_t *task;
     HRESULT hres;
 
     TRACE("(%p)->(%ld %s %p)\n", doc, nCmdexecopt, debugstr_variant(pvaIn), pvaOut);
@@ -478,14 +493,7 @@ static HRESULT exec_refresh(HTMLDocumentNode *doc, DWORD nCmdexecopt, VARIANT *p
     if(!doc->outer_window)
         return E_UNEXPECTED;
 
-    task = malloc(sizeof(*task));
-    if(!task)
-        return E_OUTOFMEMORY;
-
-    IHTMLWindow2_AddRef(&doc->outer_window->base.IHTMLWindow2_iface);
-    task->window = doc->outer_window;
-
-    return push_task(&task->header, refresh_proc, refresh_destr, doc->outer_window->task_magic);
+    return reload_page(doc->outer_window);
 }
 
 static HRESULT exec_stop(HTMLDocumentNode *doc, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
@@ -682,12 +690,19 @@ static HRESULT exec_browsemode(HTMLDocumentNode *doc, DWORD cmdexecopt, VARIANT
 
 static HRESULT exec_editmode(HTMLDocumentNode *doc, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
 {
+    HTMLDocumentObj *doc_obj;
+    HRESULT hres;
+
     TRACE("(%p)->(%08lx %p %p)\n", doc, cmdexecopt, in, out);
 
     if(in || out)
         FIXME("unsupported args\n");
 
-    return setup_edit_mode(doc->browser->doc);
+    doc_obj = doc->browser->doc;
+    IUnknown_AddRef(doc_obj->outer_unk);
+    hres = setup_edit_mode(doc_obj);
+    IUnknown_Release(doc_obj->outer_unk);
+    return hres;
 }
 
 static HRESULT exec_htmleditmode(HTMLDocumentNode *doc, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
@@ -985,8 +1000,6 @@ static HRESULT WINAPI DocObjOleCommandTarget_QueryStatus(IOleCommandTarget *ifac
 {
     HTMLDocumentObj *This = HTMLDocumentObj_from_IOleCommandTarget(iface);
 
-    if(!This->doc_node)
-        return E_UNEXPECTED;
     return IOleCommandTarget_QueryStatus(&This->doc_node->IOleCommandTarget_iface,
                                          pguidCmdGroup, cCmds, prgCmds, pCmdText);
 }
@@ -996,8 +1009,6 @@ static HRESULT WINAPI DocObjOleCommandTarget_Exec(IOleCommandTarget *iface, cons
 {
     HTMLDocumentObj *This = HTMLDocumentObj_from_IOleCommandTarget(iface);
 
-    if(!This->doc_node)
-        return E_UNEXPECTED;
     return IOleCommandTarget_Exec(&This->doc_node->IOleCommandTarget_iface,
                                   pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
 }
