@@ -23,18 +23,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "msvcrt.h"
+#include "winver.h"
+#include "imagehlp.h"
 
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 
-/* TODO:
- * - document a bit (grammar + functions)
- * - back-port this new code into tools/winedump/msmangle.c
- */
-
-/* How data types modifiers are stored:
- * M (in the following definitions) is defined for 
+/* How data types qualifiers are stored:
+ * M (in the following definitions) is defined for
  * 'A', 'B', 'C' and 'D' as follows
  *      {<A>}:  ""
  *      {<B>}:  "const "
@@ -48,9 +45,11 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
  *      in data fields:
  *              same as for arguments and also the following
  *              ?<M>x   {<M>}x
- *              
+ *
  */
 
+#define UNDNAME_NO_COMPLEX_TYPE (0x8000)
+
 struct array
 {
     unsigned            start;          /* first valid reference in array */
@@ -80,6 +79,7 @@ struct parsed_symbol
 enum datatype_e
 {
     DT_NO_LEADING_WS = 0x01,
+    DT_NO_LRSEP_WS = 0x02,
 };
 
 /* Type for parsing mangled types */
@@ -407,57 +407,67 @@ static char* get_args(struct parsed_symbol* sym, struct array* pmt_ref, BOOL z_t
     return args_str;
 }
 
-static void append_extended_modifier(struct parsed_symbol *sym, const char **where, const char *str)
+static void append_extended_qualifier(struct parsed_symbol *sym, const char **where,
+                                      const char *str, BOOL is_ms_keyword)
 {
-    if (!(sym->flags & UNDNAME_NO_MS_KEYWORDS))
+    if (!is_ms_keyword || !(sym->flags & UNDNAME_NO_MS_KEYWORDS))
     {
-        if (sym->flags & UNDNAME_NO_LEADING_UNDERSCORES)
+        if (is_ms_keyword && (sym->flags & UNDNAME_NO_LEADING_UNDERSCORES))
             str += 2;
-        *where = *where ? str_printf(sym, "%s %s", *where, str) : str;
+        *where = *where ? str_printf(sym, "%s%s%s%s", *where, is_ms_keyword ? " " : "", str, is_ms_keyword ? "" : " ") :
+            str_printf(sym, "%s%s", str, is_ms_keyword ? "" : " ");
     }
 }
 
-static void get_extended_modifier(struct parsed_symbol *sym, struct datatype_t *xdt)
+static void get_extended_qualifier(struct parsed_symbol *sym, struct datatype_t *xdt)
 {
+    unsigned fl = 0;
     xdt->left = xdt->right = NULL;
     xdt->flags = 0;
     for (;;)
     {
         switch (*sym->current)
         {
-        case 'E': append_extended_modifier(sym, &xdt->right, "__ptr64");     break;
-        case 'F': append_extended_modifier(sym, &xdt->left,  "__unaligned"); break;
-        case 'I': append_extended_modifier(sym, &xdt->right, "__restrict");  break;
-        default: return;
+        case 'E': append_extended_qualifier(sym, &xdt->right, "__ptr64", TRUE);     fl |= 2; break;
+        case 'F': append_extended_qualifier(sym, &xdt->left,  "__unaligned", TRUE); fl |= 2; break;
+#ifdef _UCRT
+        case 'G': append_extended_qualifier(sym, &xdt->right, "&", FALSE);          fl |= 1; break;
+        case 'H': append_extended_qualifier(sym, &xdt->right, "&&", FALSE);         fl |= 1; break;
+#endif
+        case 'I': append_extended_qualifier(sym, &xdt->right, "__restrict", TRUE);  fl |= 2; break;
+        default: if (fl == 1 || (fl == 3 && (sym->flags & UNDNAME_NO_MS_KEYWORDS))) xdt->flags = DT_NO_LRSEP_WS; return;
         }
         sym->current++;
     }
 }
 
 /******************************************************************
- *		get_modifier
- * Parses the type modifier. Always returns static strings.
+ *		get_qualifier
+ * Parses the type qualifier. Always returns static strings.
  */
-static BOOL get_modifier(struct parsed_symbol *sym, struct datatype_t *xdt, const char** pclass)
+static BOOL get_qualifier(struct parsed_symbol *sym, struct datatype_t *xdt, const char** pclass)
 {
     char ch;
-    const char* mod;
+    const char* qualif;
 
-    get_extended_modifier(sym, xdt);
+    get_extended_qualifier(sym, xdt);
     switch (ch = *sym->current++)
     {
-    case 'A': mod = NULL; break;
-    case 'B': mod = "const"; break;
-    case 'C': mod = "volatile"; break;
-    case 'D': mod = "const volatile"; break;
-    case 'Q': mod = NULL; break;
-    case 'R': mod = "const"; break;
-    case 'S': mod = "volatile"; break;
-    case 'T': mod = "const volatile"; break;
+    case 'A': qualif = NULL; break;
+    case 'B': qualif = "const"; break;
+    case 'C': qualif = "volatile"; break;
+    case 'D': qualif = "const volatile"; break;
+    case 'Q': qualif = NULL; break;
+    case 'R': qualif = "const"; break;
+    case 'S': qualif = "volatile"; break;
+    case 'T': qualif = "const volatile"; break;
     default: return FALSE;
     }
-    if (mod)
-        xdt->left = xdt->left ? str_printf(sym, "%s %s", mod, xdt->left) : mod;
+    if (qualif)
+    {
+        xdt->flags &= ~DT_NO_LRSEP_WS;
+        xdt->left = xdt->left ? str_printf(sym, "%s %s", qualif, xdt->left) : qualif;
+    }
     if (ch >= 'Q' && ch <= 'T') /* pointer to member, fetch class */
     {
         const char* class = get_class_name(sym);
@@ -473,48 +483,58 @@ static BOOL get_modifier(struct parsed_symbol *sym, struct datatype_t *xdt, cons
     return TRUE;
 }
 
-static BOOL get_modified_type(struct datatype_t *ct, struct parsed_symbol* sym,
-                              struct array *pmt_ref, char modif, enum datatype_flags flags)
+static BOOL get_function_qualifier(struct parsed_symbol *sym, const char** qualif)
+{
+    struct datatype_t   xdt;
+
+    if (!get_qualifier(sym, &xdt, NULL)) return FALSE;
+    *qualif = (xdt.left || xdt.right) ?
+        str_printf(sym, "%s%s%s", xdt.left, (xdt.flags & DT_NO_LRSEP_WS) ? "" : " ", xdt.right) : NULL;
+    return TRUE;
+}
+
+static BOOL get_qualified_type(struct datatype_t *ct, struct parsed_symbol* sym,
+                              struct array *pmt_ref, char qualif, enum datatype_flags flags)
 {
     struct datatype_t xdt1;
     struct datatype_t xdt2;
     const char* ref;
-    const char* str_modif;
+    const char* str_qualif;
     const char* class;
 
-    get_extended_modifier(sym, &xdt1);
+    get_extended_qualifier(sym, &xdt1);
 
-    switch (modif)
+    switch (qualif)
     {
-    case 'A': ref = " &";  str_modif = NULL;              break;
-    case 'B': ref = " &";  str_modif = " volatile";       break;
-    case 'P': ref = " *";  str_modif = NULL;              break;
-    case 'Q': ref = " *";  str_modif = " const";          break;
-    case 'R': ref = " *";  str_modif = " volatile";       break;
-    case 'S': ref = " *";  str_modif = " const volatile"; break;
-    case '?': ref = NULL;  str_modif = NULL;              break;
-    case '$': ref = " &&"; str_modif = NULL;              break;
+    case 'A': ref = " &";  str_qualif = NULL;              break;
+    case 'B': ref = " &";  str_qualif = " volatile";       break;
+    case 'P': ref = " *";  str_qualif = NULL;              break;
+    case 'Q': ref = " *";  str_qualif = " const";          break;
+    case 'R': ref = " *";  str_qualif = " volatile";       break;
+    case 'S': ref = " *";  str_qualif = " const volatile"; break;
+    case '?': ref = NULL;  str_qualif = NULL;              break;
+    case '$': ref = " &&"; str_qualif = NULL;              break;
     default: return FALSE;
     }
     ct->right = NULL;
     ct->flags = 0;
 
-    if (get_modifier(sym, &xdt2, &class))
+    if (get_qualifier(sym, &xdt2, &class))
     {
         unsigned            mark = sym->stack.num;
         struct datatype_t   sub_ct;
 
-        if (ref || str_modif || xdt1.left || xdt1.right)
+        if (ref || str_qualif || xdt1.left || xdt1.right)
         {
             if (class)
                 ct->left = str_printf(sym, "%s%s%s%s::%s%s%s",
                                       xdt1.left ? " " : NULL, xdt1.left,
                                       class ? " " : NULL, class, ref ? ref + 1 : NULL,
-                                      xdt1.right ? " " : NULL, xdt1.right, str_modif);
+                                      xdt1.right ? " " : NULL, xdt1.right, str_qualif);
             else
                 ct->left = str_printf(sym, "%s%s%s%s%s%s",
                                       xdt1.left ? " " : NULL, xdt1.left, ref,
-                                      xdt1.right ? " " : NULL, xdt1.right, str_modif);
+                                      xdt1.right ? " " : NULL, xdt1.right, str_qualif);
         }
         else
             ct->left = NULL;
@@ -543,14 +563,14 @@ static BOOL get_modified_type(struct datatype_t *ct, struct parsed_symbol* sym,
             ct->left++;
         ct->left = str_printf(sym, "%s%s%s%s%s", sub_ct.left, xdt2.left ? " " : NULL,
                               xdt2.left, ct->left,
-                              ((xdt2.left || str_modif) && (flags & WS_AFTER_QUAL_IF)) ? " " : NULL);
+                              ((xdt2.left || str_qualif) && (flags & WS_AFTER_QUAL_IF)) ? " " : NULL);
         if (sub_ct.right) ct->right = str_printf(sym, "%s%s", ct->right, sub_ct.right);
         sym->stack.num = mark;
     }
-    else if (ref || str_modif || xdt1.left || xdt1.right)
+    else if (ref || str_qualif || xdt1.left || xdt1.right)
         ct->left = str_printf(sym, "%s%s%s%s%s%s",
                               xdt1.left ? " " : NULL, xdt1.left, ref,
-                              xdt1.right ? " " : NULL, xdt1.right, str_modif);
+                              xdt1.right ? " " : NULL, xdt1.right, str_qualif);
     else
         ct->left = NULL;
     return TRUE;
@@ -926,17 +946,17 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
         }
         else
         {
-            if (!get_modified_type(ct, sym, pmt_ref, '?', flags)) goto done;
+            if (!get_qualified_type(ct, sym, pmt_ref, '?', flags)) goto done;
         }
         break;
     case 'A': /* reference */
     case 'B': /* volatile reference */
-        if (!get_modified_type(ct, sym, pmt_ref, dt, flags)) goto done;
+        if (!get_qualified_type(ct, sym, pmt_ref, dt, flags)) goto done;
         break;
     case 'Q': /* const pointer */
     case 'R': /* volatile pointer */
     case 'S': /* const volatile pointer */
-        if (!get_modified_type(ct, sym, pmt_ref, (flags & IN_ARGS) ? dt : 'P', flags)) goto done;
+        if (!get_qualified_type(ct, sym, pmt_ref, (flags & IN_ARGS) ? dt : 'P', flags)) goto done;
         break;
     case 'P': /* Pointer */
         if (isdigit(*sym->current))
@@ -949,24 +969,20 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
             {
                 struct function_signature       fs;
                 const char*                     class;
-                struct datatype_t               xdt;
+                const char*                     function_qualifier;
 
                 sym->current++;
 
                 if (!(class = get_class_name(sym)))
                     goto done;
-                if (!get_modifier(sym, &xdt, NULL))
+                if (!get_function_qualifier(sym, &function_qualifier))
                     goto done;
                 if (!get_function_signature(sym, pmt_ref, &fs))
                      goto done;
-                if (xdt.left)
-                    xdt.left = str_printf(sym, "%s %s", xdt.left, xdt.right);
-                else if (xdt.right)
-                    xdt.left = str_printf(sym, " %s", xdt.right);
 
                 ct->left  = str_printf(sym, "%s%s (%s %s::*",
                                        fs.return_ct.left, fs.return_ct.right, fs.call_conv, class);
-                ct->right = str_printf(sym, ")%s%s", fs.arguments, xdt.left);
+                ct->right = str_printf(sym, ")%s%s", fs.arguments, function_qualifier);
             }
             else if (*sym->current == '6')
             {
@@ -984,7 +1000,7 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
             }
             else goto done;
 	}
-	else if (!get_modified_type(ct, sym, pmt_ref, 'P', flags)) goto done;
+	else if (!get_qualified_type(ct, sym, pmt_ref, 'P', flags)) goto done;
         break;
     case 'W':
         if (*sym->current == '4')
@@ -1101,14 +1117,14 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
                 struct datatype_t xdt;
 
                 sym->current++;
-                if (!get_modifier(sym, &xdt, NULL)) goto done;
+                if (!get_qualifier(sym, &xdt, NULL)) goto done;
                 if (!demangle_datatype(sym, ct, pmt_ref, flags)) goto done;
                 ct->left = str_printf(sym, "%s %s", ct->left, xdt.left);
             }
             else if (*sym->current == 'Q')
             {
                 sym->current++;
-                if (!get_modified_type(ct, sym, pmt_ref, '$', flags)) goto done;
+                if (!get_qualified_type(ct, sym, pmt_ref, '$', flags)) goto done;
             }
             break;
         }
@@ -1184,7 +1200,7 @@ static BOOL handle_data(struct parsed_symbol* sym)
             str_array_init(&pmt);
 
             if (!demangle_datatype(sym, &ct, &pmt, 0)) goto done;
-            if (!get_modifier(sym, &xdt, &class)) goto done; /* class doesn't seem to be displayed */
+            if (!get_qualifier(sym, &xdt, &class)) goto done; /* class doesn't seem to be displayed */
             if (xdt.left && xdt.right) xdt.left = str_printf(sym, "%s %s", xdt.left, xdt.right);
             else if (!xdt.left) xdt.left = xdt.right;
             sym->stack.num = mark;
@@ -1193,7 +1209,7 @@ static BOOL handle_data(struct parsed_symbol* sym)
     case '6' : /* compiler generated static */
     case '7' : /* compiler generated static */
         ct.left = ct.right = NULL;
-        if (!get_modifier(sym, &xdt, NULL)) goto done;
+        if (!get_qualifier(sym, &xdt, NULL)) goto done;
         if (*sym->current != '@')
         {
             char*       cls = NULL;
@@ -1233,7 +1249,7 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
     const char*         member_type = NULL;
     struct datatype_t   ct_ret;
     const char*         call_conv;
-    struct datatype_t   xdt = {NULL};
+    const char*         function_qualifier = NULL;
     const char*         exported;
     const char*         args_str = NULL;
     const char*         name = NULL;
@@ -1362,9 +1378,7 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
                 (accmem <= 'X' && (accmem - 'A') % 8 != 2 && (accmem - 'A') % 8 != 3)))
     {
         /* Implicit 'this' pointer */
-        /* If there is an implicit this pointer, const modifier follows */
-        if (!get_modifier(sym, &xdt, NULL)) goto done;
-        if (xdt.left || xdt.right) xdt.left = str_printf(sym, "%s %s", xdt.left, xdt.right);
+        if (!get_function_qualifier(sym, &function_qualifier)) goto done;
     }
 
     if (!get_calling_convention(*sym->current++, &call_conv, &exported,
@@ -1395,8 +1409,8 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
 
     mark = sym->stack.num;
     if (has_args && !(args_str = get_args(sym, &array_pmt, TRUE, '(', ')'))) goto done;
-    if (sym->flags & UNDNAME_NAME_ONLY) args_str = xdt.left = NULL;
-    if (sym->flags & UNDNAME_NO_THISTYPE) xdt.left = NULL;
+    if (sym->flags & UNDNAME_NAME_ONLY) args_str = function_qualifier = NULL;
+    if (sym->flags & UNDNAME_NO_THISTYPE) function_qualifier = NULL;
     sym->stack.num = mark;
 
     /* Note: '()' after 'Z' means 'throws', but we don't care here
@@ -1406,7 +1420,7 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
                              access, member_type, ct_ret.left,
                              (ct_ret.left && !ct_ret.right) ? " " : NULL,
                              call_conv, call_conv ? " " : NULL, exported,
-                             name, args_str, xdt.left, ct_ret.right);
+                             name, args_str, function_qualifier, ct_ret.right);
     ret = TRUE;
 done:
     return ret;
