@@ -43,42 +43,41 @@ struct wm_char_mapping_data
     MSG  get_msg;
 };
 
-extern HMODULE user32_module DECLSPEC_HIDDEN;
+extern HMODULE user32_module;
 
 extern BOOL post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD dest_tid,
-                              DWORD type ) DECLSPEC_HIDDEN;
+                              DWORD type );
 extern BOOL unpack_dde_message( HWND hwnd, UINT message, WPARAM *wparam, LPARAM *lparam,
-                                const void *buffer, size_t size ) DECLSPEC_HIDDEN;
-extern void free_cached_data( UINT format, HANDLE handle ) DECLSPEC_HIDDEN;
-extern HANDLE render_synthesized_format( UINT format, UINT from ) DECLSPEC_HIDDEN;
+                                const void *buffer, size_t size );
+extern void free_cached_data( UINT format, HANDLE handle );
+extern HANDLE render_synthesized_format( UINT format, UINT from );
+extern void unpack_message( HWND hwnd, UINT message, WPARAM *wparam, LPARAM *lparam,
+                            void *buffer, size_t size, BOOL ansi );
 
-extern void CLIPBOARD_ReleaseOwner( HWND hwnd ) DECLSPEC_HIDDEN;
-extern HDC get_display_dc(void) DECLSPEC_HIDDEN;
-extern void release_display_dc( HDC hdc ) DECLSPEC_HIDDEN;
-extern void wait_graphics_driver_ready(void) DECLSPEC_HIDDEN;
-extern void *get_hook_proc( void *proc, const WCHAR *module, HMODULE *free_module ) DECLSPEC_HIDDEN;
-extern RECT get_virtual_screen_rect(void) DECLSPEC_HIDDEN;
-extern RECT get_primary_monitor_rect(void) DECLSPEC_HIDDEN;
-extern DWORD get_input_codepage( void ) DECLSPEC_HIDDEN;
-extern BOOL map_wparam_AtoW( UINT message, WPARAM *wparam, enum wm_char_mapping mapping ) DECLSPEC_HIDDEN;
-extern HPEN SYSCOLOR_GetPen( INT index ) DECLSPEC_HIDDEN;
-extern HBRUSH SYSCOLOR_Get55AABrush(void) DECLSPEC_HIDDEN;
-extern void SYSPARAMS_Init(void) DECLSPEC_HIDDEN;
+extern void CLIPBOARD_ReleaseOwner( HWND hwnd );
+extern HDC get_display_dc(void);
+extern void release_display_dc( HDC hdc );
+extern void *get_hook_proc( void *proc, const WCHAR *module, HMODULE *free_module );
+extern DWORD get_input_codepage( void );
+extern BOOL map_wparam_AtoW( UINT message, WPARAM *wparam, enum wm_char_mapping mapping );
+extern HPEN SYSCOLOR_GetPen( INT index );
+extern HBRUSH SYSCOLOR_Get55AABrush(void);
+extern void SYSPARAMS_Init(void);
 
 typedef LRESULT (*winproc_callback_t)( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp,
                                        LRESULT *result, void *arg );
 
 extern LRESULT WINPROC_CallProcAtoW( winproc_callback_t callback, HWND hwnd, UINT msg,
                                      WPARAM wParam, LPARAM lParam, LRESULT *result, void *arg,
-                                     enum wm_char_mapping mapping ) DECLSPEC_HIDDEN;
+                                     enum wm_char_mapping mapping );
 
-extern INT_PTR WINPROC_CallDlgProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam ) DECLSPEC_HIDDEN;
-extern INT_PTR WINPROC_CallDlgProcW( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam ) DECLSPEC_HIDDEN;
-extern void winproc_init(void) DECLSPEC_HIDDEN;
-extern void dispatch_win_proc_params( struct win_proc_params *params ) DECLSPEC_HIDDEN;
+extern INT_PTR WINPROC_CallDlgProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam );
+extern INT_PTR WINPROC_CallDlgProcW( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam );
+extern void winproc_init(void);
+extern LRESULT dispatch_win_proc_params( struct win_proc_params *params );
 
 extern ATOM get_class_info( HINSTANCE instance, const WCHAR *name, WNDCLASSEXW *info,
-                            UNICODE_STRING *name_str, BOOL ansi ) DECLSPEC_HIDDEN;
+                            UNICODE_STRING *name_str, BOOL ansi );
 
 /* kernel callbacks */
 
@@ -91,10 +90,10 @@ BOOL WINAPI User32InitBuiltinClasses( const struct win_hook_params *params, ULON
 
 /* message spy definitions */
 
-extern const char *SPY_GetMsgName( UINT msg, HWND hWnd ) DECLSPEC_HIDDEN;
-extern void SPY_EnterMessage( INT iFlag, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam ) DECLSPEC_HIDDEN;
+extern const char *SPY_GetMsgName( UINT msg, HWND hWnd );
+extern void SPY_EnterMessage( INT iFlag, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam );
 extern void SPY_ExitMessage( INT iFlag, HWND hwnd, UINT msg,
-                             LRESULT lReturn, WPARAM wParam, LPARAM lParam ) DECLSPEC_HIDDEN;
+                             LRESULT lReturn, WPARAM wParam, LPARAM lParam );
 
 #include "pshpack1.h"
 
@@ -152,28 +151,29 @@ typedef struct
 
 #include "poppack.h"
 
-extern int bitmap_info_size( const BITMAPINFO * info, WORD coloruse ) DECLSPEC_HIDDEN;
-extern BOOL get_icon_size( HICON handle, SIZE *size ) DECLSPEC_HIDDEN;
+extern int bitmap_info_size( const BITMAPINFO * info, WORD coloruse );
+extern BOOL get_icon_size( HICON handle, SIZE *size );
 
-extern struct user_api_hook *user_api DECLSPEC_HIDDEN;
-LRESULT WINAPI USER_DefDlgProc(HWND, UINT, WPARAM, LPARAM, BOOL) DECLSPEC_HIDDEN;
-LRESULT WINAPI USER_ScrollBarProc(HWND, UINT, WPARAM, LPARAM, BOOL) DECLSPEC_HIDDEN;
+extern struct user_api_hook *user_api;
+LRESULT WINAPI USER_DefDlgProc(HWND, UINT, WPARAM, LPARAM, BOOL);
+LRESULT WINAPI USER_ScrollBarProc(HWND, UINT, WPARAM, LPARAM, BOOL);
+void WINAPI USER_NonClientButtonDraw(HWND, HDC, enum NONCLIENT_BUTTON_TYPE, RECT, BOOL, BOOL);
 void WINAPI USER_ScrollBarDraw(HWND, HDC, INT, enum SCROLL_HITTEST,
                                const struct SCROLL_TRACKING_INFO *, BOOL, BOOL, RECT *, UINT,
-                               INT, INT, INT, BOOL) DECLSPEC_HIDDEN;
+                               INT, INT, INT, BOOL);
 struct scroll_info *SCROLL_GetInternalInfo( HWND hwnd, INT nBar, BOOL alloc );
 
 /* Window functions */
-BOOL is_desktop_window( HWND hwnd ) DECLSPEC_HIDDEN;
-HWND WIN_GetFullHandle( HWND hwnd ) DECLSPEC_HIDDEN;
-HWND WIN_IsCurrentProcess( HWND hwnd ) DECLSPEC_HIDDEN;
-HWND WIN_IsCurrentThread( HWND hwnd ) DECLSPEC_HIDDEN;
-ULONG WIN_SetStyle( HWND hwnd, ULONG set_bits, ULONG clear_bits ) DECLSPEC_HIDDEN;
-HWND WIN_CreateWindowEx( CREATESTRUCTW *cs, LPCWSTR className, HINSTANCE module, BOOL unicode ) DECLSPEC_HIDDEN;
-HWND *WIN_ListChildren( HWND hwnd ) DECLSPEC_HIDDEN;
-void MDI_CalcDefaultChildPos( HWND hwndClient, INT total, LPPOINT lpPos, INT delta, UINT *id ) DECLSPEC_HIDDEN;
+BOOL is_desktop_window( HWND hwnd );
+HWND WIN_GetFullHandle( HWND hwnd );
+HWND WIN_IsCurrentProcess( HWND hwnd );
+HWND WIN_IsCurrentThread( HWND hwnd );
+ULONG WIN_SetStyle( HWND hwnd, ULONG set_bits, ULONG clear_bits );
+HWND WIN_CreateWindowEx( CREATESTRUCTW *cs, LPCWSTR className, HINSTANCE module, BOOL unicode );
+HWND *WIN_ListChildren( HWND hwnd );
+void MDI_CalcDefaultChildPos( HWND hwndClient, INT total, LPPOINT lpPos, INT delta, UINT *id );
 HDESK open_winstation_desktop( HWINSTA hwinsta, LPCWSTR name, DWORD flags, BOOL inherit,
-                               ACCESS_MASK access ) DECLSPEC_HIDDEN;
+                               ACCESS_MASK access );
 
 static inline void mirror_rect( const RECT *window_rect, RECT *rect )
 {
