@@ -21,6 +21,9 @@
 #define WINE_NO_NAMELESS_EXTENSION
 
 #define EXTERN_GUID DEFINE_GUID
+
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "initguid.h"
 #include "gst_private.h"
 #include "winternl.h"
@@ -34,6 +37,8 @@ WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 DEFINE_GUID(GUID_NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 DEFINE_GUID(MEDIASUBTYPE_VC1S,MAKEFOURCC('V','C','1','S'),0x0000,0x0010,0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71);
 
+static const GUID MEDIASUBTYPE_MP3 = {0x00000055, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
+
 bool array_reserve(void **elements, size_t *capacity, size_t count, size_t size)
 {
     unsigned int new_capacity, max_capacity;
@@ -61,34 +66,34 @@ bool array_reserve(void **elements, size_t *capacity, size_t count, size_t size)
     return TRUE;
 }
 
-struct wg_parser *wg_parser_create(enum wg_parser_type type, bool unlimited_buffering)
+wg_parser_t wg_parser_create(enum wg_parser_type type, bool output_compressed)
 {
     struct wg_parser_create_params params =
     {
         .type = type,
-        .unlimited_buffering = unlimited_buffering,
+        .output_compressed = output_compressed,
         .err_on = ERR_ON(quartz),
         .warn_on = WARN_ON(quartz),
     };
 
-    TRACE("type %#x, unlimited_buffering %d.\n", type, unlimited_buffering);
+    TRACE("type %#x.\n", type);
 
     if (WINE_UNIX_CALL(unix_wg_parser_create, &params))
-        return NULL;
+        return 0;
 
-    TRACE("Returning parser %p.\n", params.parser);
+    TRACE("Returning parser %#I64x.\n", params.parser);
 
     return params.parser;
 }
 
-void wg_parser_destroy(struct wg_parser *parser)
+void wg_parser_destroy(wg_parser_t parser)
 {
-    TRACE("parser %p.\n", parser);
+    TRACE("parser %#I64x.\n", parser);
 
-    WINE_UNIX_CALL(unix_wg_parser_destroy, parser);
+    WINE_UNIX_CALL(unix_wg_parser_destroy, &parser);
 }
 
-HRESULT wg_parser_connect(struct wg_parser *parser, uint64_t file_size)
+HRESULT wg_parser_connect(wg_parser_t parser, uint64_t file_size)
 {
     struct wg_parser_connect_params params =
     {
@@ -96,26 +101,26 @@ HRESULT wg_parser_connect(struct wg_parser *parser, uint64_t file_size)
         .file_size = file_size,
     };
 
-    TRACE("parser %p, file_size %I64u.\n", parser, file_size);
+    TRACE("parser %#I64x, file_size %I64u.\n", parser, file_size);
 
     return WINE_UNIX_CALL(unix_wg_parser_connect, &params);
 }
 
-void wg_parser_disconnect(struct wg_parser *parser)
+void wg_parser_disconnect(wg_parser_t parser)
 {
-    TRACE("parser %p.\n", parser);
+    TRACE("parser %#I64x.\n", parser);
 
-    WINE_UNIX_CALL(unix_wg_parser_disconnect, parser);
+    WINE_UNIX_CALL(unix_wg_parser_disconnect, &parser);
 }
 
-bool wg_parser_get_next_read_offset(struct wg_parser *parser, uint64_t *offset, uint32_t *size)
+bool wg_parser_get_next_read_offset(wg_parser_t parser, uint64_t *offset, uint32_t *size)
 {
     struct wg_parser_get_next_read_offset_params params =
     {
         .parser = parser,
     };
 
-    TRACE("parser %p, offset %p, size %p.\n", parser, offset, size);
+    TRACE("parser %#I64x, offset %p, size %p.\n", parser, offset, size);
 
     if (WINE_UNIX_CALL(unix_wg_parser_get_next_read_offset, &params))
         return false;
@@ -124,7 +129,7 @@ bool wg_parser_get_next_read_offset(struct wg_parser *parser, uint64_t *offset,
     return true;
 }
 
-void wg_parser_push_data(struct wg_parser *parser, const void *data, uint32_t size)
+void wg_parser_push_data(wg_parser_t parser, const void *data, uint32_t size)
 {
     struct wg_parser_push_data_params params =
     {
@@ -133,25 +138,25 @@ void wg_parser_push_data(struct wg_parser *parser, const void *data, uint32_t si
         .size = size,
     };
 
-    TRACE("parser %p, data %p, size %u.\n", parser, data, size);
+    TRACE("parser %#I64x, data %p, size %u.\n", parser, data, size);
 
     WINE_UNIX_CALL(unix_wg_parser_push_data, &params);
 }
 
-uint32_t wg_parser_get_stream_count(struct wg_parser *parser)
+uint32_t wg_parser_get_stream_count(wg_parser_t parser)
 {
     struct wg_parser_get_stream_count_params params =
     {
         .parser = parser,
     };
 
-    TRACE("parser %p.\n", parser);
+    TRACE("parser %#I64x.\n", parser);
 
     WINE_UNIX_CALL(unix_wg_parser_get_stream_count, &params);
     return params.count;
 }
 
-struct wg_parser_stream *wg_parser_get_stream(struct wg_parser *parser, uint32_t index)
+wg_parser_stream_t wg_parser_get_stream(wg_parser_t parser, uint32_t index)
 {
     struct wg_parser_get_stream_params params =
     {
@@ -159,15 +164,15 @@ struct wg_parser_stream *wg_parser_get_stream(struct wg_parser *parser, uint32_t
         .index = index,
     };
 
-    TRACE("parser %p, index %u.\n", parser, index);
+    TRACE("parser %#I64x, index %u.\n", parser, index);
 
     WINE_UNIX_CALL(unix_wg_parser_get_stream, &params);
 
-    TRACE("Returning stream %p.\n", params.stream);
+    TRACE("Returning stream %#I64x.\n", params.stream);
     return params.stream;
 }
 
-void wg_parser_stream_get_preferred_format(struct wg_parser_stream *stream, struct wg_format *format)
+void wg_parser_stream_get_preferred_format(wg_parser_stream_t stream, struct wg_format *format)
 {
     struct wg_parser_stream_get_preferred_format_params params =
     {
@@ -175,12 +180,25 @@ void wg_parser_stream_get_preferred_format(struct wg_parser_stream *stream, stru
         .format = format,
     };
 
-    TRACE("stream %p, format %p.\n", stream, format);
+    TRACE("stream %#I64x, format %p.\n", stream, format);
 
     WINE_UNIX_CALL(unix_wg_parser_stream_get_preferred_format, &params);
 }
 
-void wg_parser_stream_enable(struct wg_parser_stream *stream, const struct wg_format *format)
+void wg_parser_stream_get_codec_format(wg_parser_stream_t stream, struct wg_format *format)
+{
+    struct wg_parser_stream_get_codec_format_params params =
+    {
+        .stream = stream,
+        .format = format,
+    };
+
+    TRACE("stream %#I64x, format %p.\n", stream, format);
+
+    WINE_UNIX_CALL(unix_wg_parser_stream_get_codec_format, &params);
+}
+
+void wg_parser_stream_enable(wg_parser_stream_t stream, const struct wg_format *format)
 {
     struct wg_parser_stream_enable_params params =
     {
@@ -188,19 +206,19 @@ void wg_parser_stream_enable(struct wg_parser_stream *stream, const struct wg_fo
         .format = format,
     };
 
-    TRACE("stream %p, format %p.\n", stream, format);
+    TRACE("stream %#I64x, format %p.\n", stream, format);
 
     WINE_UNIX_CALL(unix_wg_parser_stream_enable, &params);
 }
 
-void wg_parser_stream_disable(struct wg_parser_stream *stream)
+void wg_parser_stream_disable(wg_parser_stream_t stream)
 {
-    TRACE("stream %p.\n", stream);
+    TRACE("stream %#I64x.\n", stream);
 
-    WINE_UNIX_CALL(unix_wg_parser_stream_disable, stream);
+    WINE_UNIX_CALL(unix_wg_parser_stream_disable, &stream);
 }
 
-bool wg_parser_stream_get_buffer(struct wg_parser *parser, struct wg_parser_stream *stream,
+bool wg_parser_stream_get_buffer(wg_parser_t parser, wg_parser_stream_t stream,
         struct wg_parser_buffer *buffer)
 {
     struct wg_parser_stream_get_buffer_params params =
@@ -210,12 +228,12 @@ bool wg_parser_stream_get_buffer(struct wg_parser *parser, struct wg_parser_stre
         .buffer = buffer,
     };
 
-    TRACE("parser %p, stream %p, buffer %p.\n", parser, stream, buffer);
+    TRACE("parser %#I64x, stream %#I64x, buffer %p.\n", parser, stream, buffer);
 
     return !WINE_UNIX_CALL(unix_wg_parser_stream_get_buffer, &params);
 }
 
-bool wg_parser_stream_copy_buffer(struct wg_parser_stream *stream,
+bool wg_parser_stream_copy_buffer(wg_parser_stream_t stream,
         void *data, uint32_t offset, uint32_t size)
 {
     struct wg_parser_stream_copy_buffer_params params =
@@ -226,19 +244,19 @@ bool wg_parser_stream_copy_buffer(struct wg_parser_stream *stream,
         .size = size,
     };
 
-    TRACE("stream %p, data %p, offset %u, size %u.\n", stream, data, offset, size);
+    TRACE("stream %#I64x, data %p, offset %u, size %u.\n", stream, data, offset, size);
 
     return !WINE_UNIX_CALL(unix_wg_parser_stream_copy_buffer, &params);
 }
 
-void wg_parser_stream_release_buffer(struct wg_parser_stream *stream)
+void wg_parser_stream_release_buffer(wg_parser_stream_t stream)
 {
-    TRACE("stream %p.\n", stream);
+    TRACE("stream %#I64x.\n", stream);
 
-    WINE_UNIX_CALL(unix_wg_parser_stream_release_buffer, stream);
+    WINE_UNIX_CALL(unix_wg_parser_stream_release_buffer, &stream);
 }
 
-void wg_parser_stream_notify_qos(struct wg_parser_stream *stream,
+void wg_parser_stream_notify_qos(wg_parser_stream_t stream,
         bool underflow, double proportion, int64_t diff, uint64_t timestamp)
 {
     struct wg_parser_stream_notify_qos_params params =
@@ -250,20 +268,20 @@ void wg_parser_stream_notify_qos(struct wg_parser_stream *stream,
         .timestamp = timestamp,
     };
 
-    TRACE("stream %p, underflow %d, proportion %.16e, diff %I64d, timestamp %I64u.\n",
+    TRACE("stream %#I64x, underflow %d, proportion %.16e, diff %I64d, timestamp %I64u.\n",
             stream, underflow, proportion, diff, timestamp);
 
     WINE_UNIX_CALL(unix_wg_parser_stream_notify_qos, &params);
 }
 
-uint64_t wg_parser_stream_get_duration(struct wg_parser_stream *stream)
+uint64_t wg_parser_stream_get_duration(wg_parser_stream_t stream)
 {
     struct wg_parser_stream_get_duration_params params =
     {
         .stream = stream,
     };
 
-    TRACE("stream %p.\n", stream);
+    TRACE("stream %#I64x.\n", stream);
 
     WINE_UNIX_CALL(unix_wg_parser_stream_get_duration, &params);
 
@@ -271,7 +289,35 @@ uint64_t wg_parser_stream_get_duration(struct wg_parser_stream *stream)
     return params.duration;
 }
 
-void wg_parser_stream_seek(struct wg_parser_stream *stream, double rate,
+char *wg_parser_stream_get_tag(wg_parser_stream_t stream, enum wg_parser_tag tag)
+{
+    uint32_t size = 0;
+    struct wg_parser_stream_get_tag_params params =
+    {
+        .stream = stream,
+        .tag = tag,
+        .size = &size,
+    };
+    char *buffer;
+
+    if (WINE_UNIX_CALL(unix_wg_parser_stream_get_tag, &params) != STATUS_BUFFER_TOO_SMALL)
+        return NULL;
+    if (!(buffer = malloc(size)))
+    {
+        ERR("No memory.\n");
+        return NULL;
+    }
+    params.buffer = buffer;
+    if (WINE_UNIX_CALL(unix_wg_parser_stream_get_tag, &params))
+    {
+        ERR("wg_parser_stream_get_tag failed unexpectedly.\n");
+        free(buffer);
+        return NULL;
+    }
+    return buffer;
+}
+
+void wg_parser_stream_seek(wg_parser_stream_t stream, double rate,
         uint64_t start_pos, uint64_t stop_pos, DWORD start_flags, DWORD stop_flags)
 {
     struct wg_parser_stream_seek_params params =
@@ -284,38 +330,39 @@ void wg_parser_stream_seek(struct wg_parser_stream *stream, double rate,
         .stop_flags = stop_flags,
     };
 
-    TRACE("stream %p, rate %.16e, start_pos %I64u, stop_pos %I64u, start_flags %#lx, stop_flags %#lx.\n",
+    TRACE("stream %#I64x, rate %.16e, start_pos %I64u, stop_pos %I64u, start_flags %#lx, stop_flags %#lx.\n",
             stream, rate, start_pos, stop_pos, start_flags, stop_flags);
 
     WINE_UNIX_CALL(unix_wg_parser_stream_seek, &params);
 }
 
-struct wg_transform *wg_transform_create(const struct wg_format *input_format,
-        const struct wg_format *output_format)
+wg_transform_t wg_transform_create(const struct wg_format *input_format,
+        const struct wg_format *output_format, const struct wg_transform_attrs *attrs)
 {
     struct wg_transform_create_params params =
     {
         .input_format = input_format,
         .output_format = output_format,
+        .attrs = attrs,
     };
 
     TRACE("input_format %p, output_format %p.\n", input_format, output_format);
 
     if (WINE_UNIX_CALL(unix_wg_transform_create, &params))
-        return NULL;
+        return 0;
 
-    TRACE("Returning transform %p.\n", params.transform);
+    TRACE("Returning transform %#I64x.\n", params.transform);
     return params.transform;
 }
 
-void wg_transform_destroy(struct wg_transform *transform)
+void wg_transform_destroy(wg_transform_t transform)
 {
-    TRACE("transform %p.\n", transform);
+    TRACE("transform %#I64x.\n", transform);
 
-    WINE_UNIX_CALL(unix_wg_transform_destroy, transform);
+    WINE_UNIX_CALL(unix_wg_transform_destroy, &transform);
 }
 
-HRESULT wg_transform_push_data(struct wg_transform *transform, struct wg_sample *sample)
+HRESULT wg_transform_push_data(wg_transform_t transform, struct wg_sample *sample)
 {
     struct wg_transform_push_data_params params =
     {
@@ -324,7 +371,7 @@ HRESULT wg_transform_push_data(struct wg_transform *transform, struct wg_sample
     };
     NTSTATUS status;
 
-    TRACE("transform %p, sample %p.\n", transform, sample);
+    TRACE("transform %#I64x, sample %p.\n", transform, sample);
 
     if ((status = WINE_UNIX_CALL(unix_wg_transform_push_data, &params)))
         return HRESULT_FROM_NT(status);
@@ -332,7 +379,7 @@ HRESULT wg_transform_push_data(struct wg_transform *transform, struct wg_sample
     return params.result;
 }
 
-HRESULT wg_transform_read_data(struct wg_transform *transform, struct wg_sample *sample,
+HRESULT wg_transform_read_data(wg_transform_t transform, struct wg_sample *sample,
         struct wg_format *format)
 {
     struct wg_transform_read_data_params params =
@@ -343,7 +390,7 @@ HRESULT wg_transform_read_data(struct wg_transform *transform, struct wg_sample
     };
     NTSTATUS status;
 
-    TRACE("transform %p, sample %p, format %p.\n", transform, sample, format);
+    TRACE("transform %#I64x, sample %p, format %p.\n", transform, sample, format);
 
     if ((status = WINE_UNIX_CALL(unix_wg_transform_read_data, &params)))
         return HRESULT_FROM_NT(status);
@@ -351,7 +398,23 @@ HRESULT wg_transform_read_data(struct wg_transform *transform, struct wg_sample
     return params.result;
 }
 
-bool wg_transform_set_output_format(struct wg_transform *transform, struct wg_format *format)
+bool wg_transform_get_status(wg_transform_t transform, bool *accepts_input)
+{
+    struct wg_transform_get_status_params params =
+    {
+        .transform = transform,
+    };
+
+    TRACE("transform %#I64x, accepts_input %p.\n", transform, accepts_input);
+
+    if (WINE_UNIX_CALL(unix_wg_transform_get_status, &params))
+        return false;
+
+    *accepts_input = params.accepts_input;
+    return true;
+}
+
+bool wg_transform_set_output_format(wg_transform_t transform, struct wg_format *format)
 {
     struct wg_transform_set_output_format_params params =
     {
@@ -359,11 +422,215 @@ bool wg_transform_set_output_format(struct wg_transform *transform, struct wg_fo
         .format = format,
     };
 
-    TRACE("transform %p, format %p.\n", transform, format);
+    TRACE("transform %#I64x, format %p.\n", transform, format);
 
     return !WINE_UNIX_CALL(unix_wg_transform_set_output_format, &params);
 }
 
+HRESULT wg_transform_drain(wg_transform_t transform)
+{
+    NTSTATUS status;
+
+    TRACE("transform %#I64x.\n", transform);
+
+    if ((status = WINE_UNIX_CALL(unix_wg_transform_drain, &transform)))
+    {
+        WARN("wg_transform_drain returned status %#lx\n", status);
+        return HRESULT_FROM_NT(status);
+    }
+
+    return S_OK;
+}
+
+HRESULT wg_transform_flush(wg_transform_t transform)
+{
+    NTSTATUS status;
+
+    TRACE("transform %#I64x.\n", transform);
+
+    if ((status = WINE_UNIX_CALL(unix_wg_transform_flush, &transform)))
+    {
+        WARN("wg_transform_flush returned status %#lx\n", status);
+        return HRESULT_FROM_NT(status);
+    }
+
+    return S_OK;
+}
+
+HRESULT wg_muxer_create(const char *format, wg_muxer_t *muxer)
+{
+    struct wg_muxer_create_params params =
+    {
+        .format = format,
+    };
+    NTSTATUS status;
+
+    TRACE("format %p, muxer %p.\n", format, muxer);
+
+    if (!(status = WINE_UNIX_CALL(unix_wg_muxer_create, &params)))
+    {
+        *muxer = params.muxer;
+        TRACE("Created wg_muxer %#I64x.\n", params.muxer);
+    }
+    else
+    {
+        WARN("Failed to create muxer, status %#lx.\n", status);
+    }
+
+    return HRESULT_FROM_NT(status);
+}
+
+void wg_muxer_destroy(wg_muxer_t muxer)
+{
+    TRACE("muxer %#I64x.\n", muxer);
+
+    WINE_UNIX_CALL(unix_wg_muxer_destroy, &muxer);
+}
+
+HRESULT wg_muxer_add_stream(wg_muxer_t muxer, UINT32 stream_id, const struct wg_format *format)
+{
+    struct wg_muxer_add_stream_params params =
+    {
+        .muxer = muxer,
+        .stream_id = stream_id,
+        .format = format,
+    };
+    NTSTATUS status;
+
+    TRACE("muxer %#I64x, stream_id %u, format %p.\n", muxer, stream_id, format);
+
+    if ((status = WINE_UNIX_CALL(unix_wg_muxer_add_stream, &params)))
+    {
+        WARN("Failed to add stream, status %#lx.\n", status);
+        return HRESULT_FROM_NT(status);
+    }
+
+    return S_OK;
+}
+
+HRESULT wg_muxer_start(wg_muxer_t muxer)
+{
+    NTSTATUS status;
+
+    TRACE("muxer %#I64x.\n",  muxer);
+
+    if ((status = WINE_UNIX_CALL(unix_wg_muxer_start, &muxer)))
+    {
+        WARN("Failed to start muxer, status %#lx.\n", status);
+        return HRESULT_FROM_NT(status);
+    }
+
+    return S_OK;
+}
+
+HRESULT wg_muxer_push_sample(wg_muxer_t muxer, struct wg_sample *sample, UINT32 steam_id)
+{
+    struct wg_muxer_push_sample_params params =
+    {
+        .muxer = muxer,
+        .sample = sample,
+        .stream_id = steam_id,
+    };
+    NTSTATUS status;
+
+    TRACE("muxer %#I64x, sample %p.\n", muxer, sample);
+
+    if ((status = WINE_UNIX_CALL(unix_wg_muxer_push_sample, &params)))
+    {
+        WARN("Failed to push sample, status %#lx.\n", status);
+        return HRESULT_FROM_NT(status);
+    }
+
+    return S_OK;
+}
+
+HRESULT wg_muxer_read_data(wg_muxer_t muxer, void *buffer, UINT32 *size, UINT64 *offset)
+{
+    struct wg_muxer_read_data_params params =
+    {
+        .muxer = muxer,
+        .buffer = buffer,
+        .size = *size,
+        .offset = UINT64_MAX,
+    };
+    NTSTATUS status;
+
+    TRACE("muxer %#I64x, buffer %p, size %u.\n", muxer, buffer, *size);
+
+    if (SUCCEEDED(status = WINE_UNIX_CALL(unix_wg_muxer_read_data, &params)))
+    {
+        *size = params.size;
+        *offset = params.offset;
+        TRACE("Read %u bytes, offset %#I64x.\n", *size, *offset);
+    }
+
+    return HRESULT_FROM_NT(status);
+}
+
+#define ALIGN(n, alignment) (((n) + (alignment) - 1) & ~((alignment) - 1))
+
+unsigned int wg_format_get_stride(const struct wg_format *format)
+{
+    const unsigned int width = format->u.video.width;
+
+    switch (format->u.video.format)
+    {
+        case WG_VIDEO_FORMAT_AYUV:
+            return width * 4;
+
+        case WG_VIDEO_FORMAT_BGRA:
+        case WG_VIDEO_FORMAT_BGRx:
+            return width * 4;
+
+        case WG_VIDEO_FORMAT_BGR:
+            return ALIGN(width * 3, 4);
+
+        case WG_VIDEO_FORMAT_UYVY:
+        case WG_VIDEO_FORMAT_YUY2:
+        case WG_VIDEO_FORMAT_YVYU:
+            return ALIGN(width * 2, 4);
+
+        case WG_VIDEO_FORMAT_RGB15:
+        case WG_VIDEO_FORMAT_RGB16:
+            return ALIGN(width * 2, 4);
+
+        case WG_VIDEO_FORMAT_I420:
+        case WG_VIDEO_FORMAT_NV12:
+        case WG_VIDEO_FORMAT_YV12:
+            return ALIGN(width, 4); /* Y plane */
+
+        case WG_VIDEO_FORMAT_UNKNOWN:
+            FIXME("Cannot calculate stride for unknown video format.\n");
+    }
+
+    return 0;
+}
+
+bool wg_video_format_is_rgb(enum wg_video_format format)
+{
+    switch (format)
+    {
+        case WG_VIDEO_FORMAT_BGRA:
+        case WG_VIDEO_FORMAT_BGRx:
+        case WG_VIDEO_FORMAT_BGR:
+        case WG_VIDEO_FORMAT_RGB15:
+        case WG_VIDEO_FORMAT_RGB16:
+            return true;
+
+        case WG_VIDEO_FORMAT_AYUV:
+        case WG_VIDEO_FORMAT_I420:
+        case WG_VIDEO_FORMAT_NV12:
+        case WG_VIDEO_FORMAT_UYVY:
+        case WG_VIDEO_FORMAT_YUY2:
+        case WG_VIDEO_FORMAT_YV12:
+        case WG_VIDEO_FORMAT_YVYU:
+        case WG_VIDEO_FORMAT_UNKNOWN:
+            break;
+    }
+
+    return false;
+}
+
 BOOL WINAPI DllMain(HINSTANCE instance, DWORD reason, void *reserved)
 {
     if (reason == DLL_PROCESS_ATTACH)
@@ -449,6 +716,7 @@ static const IClassFactoryVtbl class_factory_vtbl =
 static struct class_factory avi_splitter_cf = {{&class_factory_vtbl}, avi_splitter_create};
 static struct class_factory decodebin_parser_cf = {{&class_factory_vtbl}, decodebin_parser_create};
 static struct class_factory mpeg_audio_codec_cf = {{&class_factory_vtbl}, mpeg_audio_codec_create};
+static struct class_factory mpeg_video_codec_cf = {{&class_factory_vtbl}, mpeg_video_codec_create};
 static struct class_factory mpeg_layer3_decoder_cf = {{&class_factory_vtbl}, mpeg_layer3_decoder_create};
 static struct class_factory mpeg_splitter_cf = {{&class_factory_vtbl}, mpeg_splitter_create};
 static struct class_factory wave_parser_cf = {{&class_factory_vtbl}, wave_parser_create};
@@ -456,6 +724,8 @@ static struct class_factory wma_decoder_cf = {{&class_factory_vtbl}, wma_decoder
 static struct class_factory wmv_decoder_cf = {{&class_factory_vtbl}, wmv_decoder_create};
 static struct class_factory resampler_cf = {{&class_factory_vtbl}, resampler_create};
 static struct class_factory color_convert_cf = {{&class_factory_vtbl}, color_convert_create};
+static struct class_factory mp3_sink_class_factory_cf = {{&class_factory_vtbl}, mp3_sink_class_factory_create};
+static struct class_factory mpeg4_sink_class_factory_cf = {{&class_factory_vtbl}, mpeg4_sink_class_factory_create};
 
 HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, void **out)
 {
@@ -476,6 +746,8 @@ HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, void **out)
         factory = &decodebin_parser_cf;
     else if (IsEqualGUID(clsid, &CLSID_CMpegAudioCodec))
         factory = &mpeg_audio_codec_cf;
+    else if (IsEqualGUID(clsid, &CLSID_CMpegVideoCodec))
+        factory = &mpeg_video_codec_cf;
     else if (IsEqualGUID(clsid, &CLSID_mpeg_layer3_decoder))
         factory = &mpeg_layer3_decoder_cf;
     else if (IsEqualGUID(clsid, &CLSID_MPEG1Splitter))
@@ -490,6 +762,10 @@ HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, void **out)
         factory = &resampler_cf;
     else if (IsEqualGUID(clsid, &CLSID_CColorConvertDMO))
         factory = &color_convert_cf;
+    else if (IsEqualGUID(clsid, &CLSID_MFMP3SinkClassFactory))
+        factory = &mp3_sink_class_factory_cf;
+    else if (IsEqualGUID(clsid, &CLSID_MFMPEG4SinkClassFactory))
+        factory = &mpeg4_sink_class_factory_cf;
     else
     {
         FIXME("%s not implemented, returning CLASS_E_CLASSNOTAVAILABLE.\n", debugstr_guid(clsid));
@@ -503,6 +779,9 @@ static BOOL CALLBACK init_gstreamer_proc(INIT_ONCE *once, void *param, void **ct
 {
     HINSTANCE handle;
 
+    if (WINE_UNIX_CALL(unix_wg_init_gstreamer, NULL))
+        return FALSE;
+
     /* Unloading glib is a bad idea.. it installs atexit handlers,
      * so never unload the dll after loading */
     GetModuleHandleExW(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_PIN,
@@ -582,6 +861,69 @@ static const REGFILTER2 reg_mpeg_audio_codec =
     .u.s2.rgPins2 = reg_mpeg_audio_codec_pins,
 };
 
+static const REGPINTYPES reg_mpeg_video_codec_sink_mts[2] =
+{
+    {&MEDIATYPE_Video, &MEDIASUBTYPE_MPEG1Packet},
+    {&MEDIATYPE_Video, &MEDIASUBTYPE_MPEG1Payload},
+};
+
+static const REGPINTYPES reg_mpeg_video_codec_source_mts[1] =
+{
+    {&MEDIATYPE_Video, &GUID_NULL},
+};
+
+static const REGFILTERPINS2 reg_mpeg_video_codec_pins[2] =
+{
+    {
+        .nMediaTypes = 2,
+        .lpMediaType = reg_mpeg_video_codec_sink_mts,
+    },
+    {
+        .dwFlags = REG_PINFLAG_B_OUTPUT,
+        .nMediaTypes = 1,
+        .lpMediaType = reg_mpeg_video_codec_source_mts,
+    },
+};
+
+static const REGFILTER2 reg_mpeg_video_codec =
+{
+    .dwVersion = 2,
+    .dwMerit = 0x40000001,
+    .u.s2.cPins2 = 2,
+    .u.s2.rgPins2 = reg_mpeg_video_codec_pins,
+};
+
+static const REGPINTYPES reg_mpeg_layer3_decoder_sink_mts[1] =
+{
+    {&MEDIATYPE_Audio, &MEDIASUBTYPE_MP3},
+};
+
+static const REGPINTYPES reg_mpeg_layer3_decoder_source_mts[1] =
+{
+    {&MEDIATYPE_Audio, &MEDIASUBTYPE_PCM},
+};
+
+static const REGFILTERPINS2 reg_mpeg_layer3_decoder_pins[2] =
+{
+    {
+        .nMediaTypes = 1,
+        .lpMediaType = reg_mpeg_layer3_decoder_sink_mts,
+    },
+    {
+        .dwFlags = REG_PINFLAG_B_OUTPUT,
+        .nMediaTypes = 1,
+        .lpMediaType = reg_mpeg_layer3_decoder_source_mts,
+    },
+};
+
+static const REGFILTER2 reg_mpeg_layer3_decoder =
+{
+    .dwVersion = 2,
+    .dwMerit = 0x00810000,
+    .u.s2.cPins2 = 2,
+    .u.s2.rgPins2 = reg_mpeg_layer3_decoder_pins,
+};
+
 static const REGPINTYPES reg_mpeg_splitter_sink_mts[4] =
 {
     {&MEDIATYPE_Stream, &MEDIASUBTYPE_MPEG1Audio},
@@ -786,6 +1128,10 @@ HRESULT WINAPI DllRegisterServer(void)
             L"GStreamer splitter filter", NULL, NULL, NULL, &reg_decodebin_parser);
     IFilterMapper2_RegisterFilter(mapper, &CLSID_CMpegAudioCodec,
             L"MPEG Audio Decoder", NULL, NULL, NULL, &reg_mpeg_audio_codec);
+    IFilterMapper2_RegisterFilter(mapper, &CLSID_CMpegVideoCodec,
+            L"MPEG Video Decoder", NULL, NULL, NULL, &reg_mpeg_video_codec);
+    IFilterMapper2_RegisterFilter(mapper, &CLSID_mpeg_layer3_decoder,
+            L"MPEG Layer-3 Decoder", NULL, NULL, NULL, &reg_mpeg_layer3_decoder);
     IFilterMapper2_RegisterFilter(mapper, &CLSID_MPEG1Splitter,
             L"MPEG-I Stream Splitter", NULL, NULL, NULL, &reg_mpeg_splitter);
     IFilterMapper2_RegisterFilter(mapper, &CLSID_WAVEParser, L"Wave Parser", NULL, NULL, NULL, &reg_wave_parser);
@@ -825,6 +1171,8 @@ HRESULT WINAPI DllUnregisterServer(void)
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_AviSplitter);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_decodebin_parser);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_CMpegAudioCodec);
+    IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_CMpegVideoCodec);
+    IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_mpeg_layer3_decoder);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_MPEG1Splitter);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_WAVEParser);
 
