@@ -40,7 +40,7 @@ struct transform
     IQualityControl source_IQualityControl_iface;
     IQualityControl *qc_sink;
 
-    struct wg_transform *transform;
+    wg_transform_t transform;
     struct wg_sample_queue *sample_queue;
 
     const struct transform_ops *ops;
@@ -78,7 +78,6 @@ static void transform_destroy(struct strmbase_filter *iface)
     strmbase_sink_cleanup(&filter->sink);
     strmbase_filter_cleanup(&filter->filter);
 
-    wg_sample_queue_destroy(filter->sample_queue);
     free(filter);
 }
 
@@ -99,6 +98,7 @@ static HRESULT transform_init_stream(struct strmbase_filter *iface)
 {
     struct transform *filter = impl_from_strmbase_filter(iface);
     struct wg_format input_format, output_format;
+    struct wg_transform_attrs attrs = {0};
     HRESULT hr;
 
     if (filter->source.pin.peer)
@@ -109,9 +109,15 @@ static HRESULT transform_init_stream(struct strmbase_filter *iface)
         if (!amt_to_wg_format(&filter->source.pin.mt, &output_format))
             return E_FAIL;
 
-        filter->transform = wg_transform_create(&input_format, &output_format);
+        if (FAILED(hr = wg_sample_queue_create(&filter->sample_queue)))
+            return hr;
+
+        filter->transform = wg_transform_create(&input_format, &output_format, &attrs);
         if (!filter->transform)
+        {
+            wg_sample_queue_destroy(filter->sample_queue);
             return E_FAIL;
+        }
 
         hr = IMemAllocator_Commit(filter->source.pAllocator);
         if (FAILED(hr))
@@ -129,7 +135,10 @@ static HRESULT transform_cleanup_stream(struct strmbase_filter *iface)
     {
         IMemAllocator_Decommit(filter->source.pAllocator);
 
+        EnterCriticalSection(&filter->filter.stream_cs);
         wg_transform_destroy(filter->transform);
+        wg_sample_queue_destroy(filter->sample_queue);
+        LeaveCriticalSection(&filter->filter.stream_cs);
     }
 
     return S_OK;
@@ -536,18 +545,11 @@ static const IQualityControlVtbl source_quality_control_vtbl =
 static HRESULT transform_create(IUnknown *outer, const CLSID *clsid, const struct transform_ops *ops, struct transform **out)
 {
     struct transform *object;
-    HRESULT hr;
 
     object = calloc(1, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
-    if (FAILED(hr = wg_sample_queue_create(&object->sample_queue)))
-    {
-        free(object);
-        return hr;
-    }
-
     strmbase_filter_init(&object->filter, outer, clsid, &filter_ops);
     strmbase_sink_init(&object->sink, &object->filter, L"In", &sink_ops, NULL);
     strmbase_source_init(&object->source, &object->filter, L"Out", &source_ops);
@@ -709,11 +711,12 @@ HRESULT mpeg_audio_codec_create(IUnknown *outer, IUnknown **out)
             .rate = 44100,
         },
     };
-    struct wg_transform *transform;
+    struct wg_transform_attrs attrs = {0};
+    wg_transform_t transform;
     struct transform *object;
     HRESULT hr;
 
-    transform = wg_transform_create(&input_format, &output_format);
+    transform = wg_transform_create(&input_format, &output_format, &attrs);
     if (!transform)
     {
         ERR_(winediag)("GStreamer doesn't support MPEG-1 audio decoding, please install appropriate plugins.\n");
@@ -735,6 +738,147 @@ HRESULT mpeg_audio_codec_create(IUnknown *outer, IUnknown **out)
     return hr;
 }
 
+static HRESULT mpeg_video_codec_sink_query_accept(struct transform *filter, const AM_MEDIA_TYPE *mt)
+{
+    if (!IsEqualGUID(&mt->majortype, &MEDIATYPE_Video)
+            || !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1Payload)
+            || !IsEqualGUID(&mt->formattype, &FORMAT_MPEGVideo)
+            || mt->cbFormat < sizeof(MPEG1VIDEOINFO))
+        return S_FALSE;
+
+    return S_OK;
+}
+
+static HRESULT mpeg_video_codec_source_query_accept(struct transform *filter, const AM_MEDIA_TYPE *mt)
+{
+    if (!filter->sink.pin.peer)
+        return S_FALSE;
+
+    if (!IsEqualGUID(&mt->majortype, &MEDIATYPE_Video)
+            || !IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo)
+            || mt->cbFormat < sizeof(VIDEOINFOHEADER))
+        return S_FALSE;
+
+    if (!IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_YV12)
+            /* missing: MEDIASUBTYPE_Y41P, not supported by GStreamer */
+            && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_YUY2)
+            && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_UYVY)
+            && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB24)
+            && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB32)
+            && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB565)
+            && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB555)
+            /* missing: MEDIASUBTYPE_RGB8, not supported by GStreamer */)
+        return S_FALSE;
+
+    return S_OK;
+}
+
+static HRESULT mpeg_video_codec_source_get_media_type(struct transform *filter, unsigned int index, AM_MEDIA_TYPE *mt)
+{
+    static const enum wg_video_format formats[] = {
+        WG_VIDEO_FORMAT_YV12,
+        WG_VIDEO_FORMAT_YUY2,
+        WG_VIDEO_FORMAT_UYVY,
+        WG_VIDEO_FORMAT_BGR,
+        WG_VIDEO_FORMAT_BGRx,
+        WG_VIDEO_FORMAT_RGB16,
+        WG_VIDEO_FORMAT_RGB15,
+    };
+
+    const MPEG1VIDEOINFO *input_format = (MPEG1VIDEOINFO*)filter->sink.pin.mt.pbFormat;
+    struct wg_format wg_format = {};
+    VIDEOINFO *video_format;
+
+    if (!filter->sink.pin.peer)
+        return VFW_S_NO_MORE_ITEMS;
+
+    if (index >= ARRAY_SIZE(formats))
+        return VFW_S_NO_MORE_ITEMS;
+
+    input_format = (MPEG1VIDEOINFO*)filter->sink.pin.mt.pbFormat;
+    wg_format.major_type = WG_MAJOR_TYPE_VIDEO;
+    wg_format.u.video.format = formats[index];
+    wg_format.u.video.width = input_format->hdr.bmiHeader.biWidth;
+    wg_format.u.video.height = input_format->hdr.bmiHeader.biHeight;
+    wg_format.u.video.fps_n = 10000000;
+    wg_format.u.video.fps_d = input_format->hdr.AvgTimePerFrame;
+    if (!amt_from_wg_format(mt, &wg_format, false))
+        return E_OUTOFMEMORY;
+
+    video_format = (VIDEOINFO*)mt->pbFormat;
+    video_format->bmiHeader.biHeight = abs(video_format->bmiHeader.biHeight);
+    SetRect(&video_format->rcSource, 0, 0, video_format->bmiHeader.biWidth, video_format->bmiHeader.biHeight);
+
+    video_format->bmiHeader.biXPelsPerMeter = 2000;
+    video_format->bmiHeader.biYPelsPerMeter = 2000;
+    video_format->dwBitRate = MulDiv(video_format->bmiHeader.biSizeImage * 8, 10000000, video_format->AvgTimePerFrame);
+    mt->lSampleSize = video_format->bmiHeader.biSizeImage;
+    mt->bTemporalCompression = FALSE;
+    mt->bFixedSizeSamples = TRUE;
+
+    return S_OK;
+}
+
+static HRESULT mpeg_video_codec_source_decide_buffer_size(struct transform *filter, IMemAllocator *allocator, ALLOCATOR_PROPERTIES *props)
+{
+    VIDEOINFOHEADER *output_format = (VIDEOINFOHEADER *)filter->source.pin.mt.pbFormat;
+    ALLOCATOR_PROPERTIES ret_props;
+
+    props->cBuffers = max(props->cBuffers, 1);
+    props->cbBuffer = max(props->cbBuffer, output_format->bmiHeader.biSizeImage);
+    props->cbAlign = max(props->cbAlign, 1);
+
+    return IMemAllocator_SetProperties(allocator, props, &ret_props);
+}
+
+static const struct transform_ops mpeg_video_codec_transform_ops =
+{
+    mpeg_video_codec_sink_query_accept,
+    mpeg_video_codec_source_query_accept,
+    mpeg_video_codec_source_get_media_type,
+    mpeg_video_codec_source_decide_buffer_size,
+};
+
+HRESULT mpeg_video_codec_create(IUnknown *outer, IUnknown **out)
+{
+    static const struct wg_format output_format =
+    {
+        .major_type = WG_MAJOR_TYPE_VIDEO,
+        .u.video = {
+            .format = WG_VIDEO_FORMAT_I420,
+            /* size doesn't matter, this one is only used to check if the GStreamer plugin exists */
+        },
+    };
+    static const struct wg_format input_format =
+    {
+        .major_type = WG_MAJOR_TYPE_VIDEO_MPEG1,
+        .u.video_mpeg1 = {},
+    };
+    struct wg_transform_attrs attrs = {0};
+    wg_transform_t transform;
+    struct transform *object;
+    HRESULT hr;
+
+    transform = wg_transform_create(&input_format, &output_format, &attrs);
+    if (!transform)
+    {
+        ERR_(winediag)("GStreamer doesn't support MPEG-1 video decoding, please install appropriate plugins.\n");
+        return E_FAIL;
+    }
+    wg_transform_destroy(transform);
+
+    hr = transform_create(outer, &CLSID_CMpegVideoCodec, &mpeg_video_codec_transform_ops, &object);
+    if (FAILED(hr))
+        return hr;
+
+    wcscpy(object->sink.pin.name, L"Input");
+    wcscpy(object->source.pin.name, L"Output");
+
+    TRACE("Created MPEG video decoder %p.\n", object);
+    *out = &object->filter.IUnknown_inner;
+    return hr;
+}
+
 static HRESULT mpeg_layer3_decoder_sink_query_accept(struct transform *filter, const AM_MEDIA_TYPE *mt)
 {
     const MPEGLAYER3WAVEFORMAT *format;
@@ -766,12 +910,50 @@ static HRESULT mpeg_layer3_decoder_source_query_accept(struct transform *filter,
 
 static HRESULT mpeg_layer3_decoder_source_get_media_type(struct transform *filter, unsigned int index, AM_MEDIA_TYPE *mt)
 {
-    return VFW_S_NO_MORE_ITEMS;
+    const MPEGLAYER3WAVEFORMAT *input_format;
+    WAVEFORMATEX *output_format;
+
+    if (!filter->sink.pin.peer)
+        return VFW_S_NO_MORE_ITEMS;
+
+    if (index > 0)
+        return VFW_S_NO_MORE_ITEMS;
+
+    input_format = (const MPEGLAYER3WAVEFORMAT *)filter->sink.pin.mt.pbFormat;
+
+    output_format = CoTaskMemAlloc(sizeof(*output_format));
+    if (!output_format)
+        return E_OUTOFMEMORY;
+
+    memset(output_format, 0, sizeof(*output_format));
+    output_format->wFormatTag = WAVE_FORMAT_PCM;
+    output_format->nSamplesPerSec = input_format->wfx.nSamplesPerSec;
+    output_format->nChannels = input_format->wfx.nChannels;
+    output_format->wBitsPerSample = 16;
+    output_format->nBlockAlign = output_format->nChannels * output_format->wBitsPerSample / 8;
+    output_format->nAvgBytesPerSec = output_format->nBlockAlign * output_format->nSamplesPerSec;
+
+    memset(mt, 0, sizeof(*mt));
+    mt->majortype = MEDIATYPE_Audio;
+    mt->subtype = MEDIASUBTYPE_PCM;
+    mt->bFixedSizeSamples = TRUE;
+    mt->lSampleSize = 1152 * output_format->nBlockAlign;
+    mt->formattype = FORMAT_WaveFormatEx;
+    mt->cbFormat = sizeof(*output_format);
+    mt->pbFormat = (BYTE *)output_format;
+
+    return S_OK;
 }
 
 static HRESULT mpeg_layer3_decoder_source_decide_buffer_size(struct transform *filter, IMemAllocator *allocator, ALLOCATOR_PROPERTIES *props)
 {
-    return S_OK;
+    ALLOCATOR_PROPERTIES ret_props;
+
+    props->cBuffers = max(props->cBuffers, 8);
+    props->cbBuffer = max(props->cbBuffer, filter->source.pin.mt.lSampleSize * 4);
+    props->cbAlign = max(props->cbAlign, 1);
+
+    return IMemAllocator_SetProperties(allocator, props, &ret_props);
 }
 
 static const struct transform_ops mpeg_layer3_decoder_transform_ops =
@@ -784,9 +966,40 @@ static const struct transform_ops mpeg_layer3_decoder_transform_ops =
 
 HRESULT mpeg_layer3_decoder_create(IUnknown *outer, IUnknown **out)
 {
+    static const struct wg_format output_format =
+    {
+        .major_type = WG_MAJOR_TYPE_AUDIO,
+        .u.audio =
+        {
+            .format = WG_AUDIO_FORMAT_S16LE,
+            .channel_mask = 1,
+            .channels = 1,
+            .rate = 44100,
+        },
+    };
+    static const struct wg_format input_format =
+    {
+        .major_type = WG_MAJOR_TYPE_AUDIO_MPEG1,
+        .u.audio_mpeg1 =
+        {
+            .layer = 3,
+            .channels = 1,
+            .rate = 44100,
+        },
+    };
+    struct wg_transform_attrs attrs = {0};
+    wg_transform_t transform;
     struct transform *object;
     HRESULT hr;
 
+    transform = wg_transform_create(&input_format, &output_format, &attrs);
+    if (!transform)
+    {
+        ERR_(winediag)("GStreamer doesn't support MPEG-1 audio decoding, please install appropriate plugins.\n");
+        return E_FAIL;
+    }
+    wg_transform_destroy(transform);
+
     hr = transform_create(outer, &CLSID_mpeg_layer3_decoder, &mpeg_layer3_decoder_transform_ops, &object);
     if (FAILED(hr))
         return hr;
