@@ -37,13 +37,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(macdrv);
 
-#ifndef kIOPMAssertionTypePreventUserIdleDisplaySleep
-#define kIOPMAssertionTypePreventUserIdleDisplaySleep CFSTR("PreventUserIdleDisplaySleep")
-#endif
-#ifndef kCFCoreFoundationVersionNumber10_7
-#define kCFCoreFoundationVersionNumber10_7      635.00
-#endif
-
 #define IS_OPTION_TRUE(ch) \
     ((ch) == 'y' || (ch) == 'Y' || (ch) == 't' || (ch) == 'T' || (ch) == '1')
 
@@ -138,13 +131,13 @@ HKEY open_hkcu_key(const char *name)
             return 0;
 
         sid = ((TOKEN_USER *)sid_data)->User.Sid;
-        len = sprintf(buffer, "\\Registry\\User\\S-%u-%u", sid->Revision,
-                       MAKELONG(MAKEWORD(sid->IdentifierAuthority.Value[5],
-                                         sid->IdentifierAuthority.Value[4]),
-                                MAKEWORD(sid->IdentifierAuthority.Value[3],
-                                         sid->IdentifierAuthority.Value[2])));
+        len = snprintf(buffer, sizeof(buffer), "\\Registry\\User\\S-%u-%u", sid->Revision,
+                      (unsigned int)MAKELONG(MAKEWORD(sid->IdentifierAuthority.Value[5],
+                                                      sid->IdentifierAuthority.Value[4]),
+                                             MAKEWORD(sid->IdentifierAuthority.Value[3],
+                                                      sid->IdentifierAuthority.Value[2])));
         for (i = 0; i < sid->SubAuthorityCount; i++)
-            len += sprintf(buffer + len, "-%u", sid->SubAuthority[i]);
+            len += snprintf(buffer + len, sizeof(buffer) - len, "-%u", (unsigned int)sid->SubAuthority[i]);
 
         ascii_to_unicode(bufferW, buffer, len);
         hkcu = reg_open_key(NULL, bufferW, len * sizeof(WCHAR));
@@ -390,6 +383,8 @@ static void setup_options(void)
     if (!get_config_key(hkey, NULL, "RetinaMode", buffer, sizeof(buffer)))
         retina_enabled = IS_OPTION_TRUE(buffer[0]);
 
+    retina_on = retina_enabled;
+
     if (appkey) NtClose(appkey);
     if (hkey) NtClose(hkey);
 }
@@ -593,13 +588,7 @@ BOOL macdrv_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param,
             }
             else if (powerAssertion == kIOPMNullAssertionID)
             {
-                CFStringRef assertName;
-                /*Are we running Lion or later?*/
-                if (kCFCoreFoundationVersionNumber >= kCFCoreFoundationVersionNumber10_7)
-                    assertName = kIOPMAssertionTypePreventUserIdleDisplaySleep;
-                else
-                    assertName = kIOPMAssertionTypeNoDisplaySleep;
-                IOPMAssertionCreateWithName( assertName, kIOPMAssertionLevelOn,
+                IOPMAssertionCreateWithName( kIOPMAssertionTypePreventUserIdleDisplaySleep, kIOPMAssertionLevelOn,
                                              CFSTR("Wine Process requesting no screen saver"),
                                              &powerAssertion);
             }
@@ -618,19 +607,6 @@ NTSTATUS macdrv_client_func(enum macdrv_client_funcs id, const void *params, ULO
 }
 
 
-static NTSTATUS macdrv_ime_clear(void *arg)
-{
-    macdrv_clear_ime_text();
-    return 0;
-}
-
-
-static NTSTATUS macdrv_ime_using_input_method(void *arg)
-{
-    return macdrv_using_input_method();
-}
-
-
 static NTSTATUS macdrv_quit_result(void *arg)
 {
     struct quit_result_params *params = arg;
@@ -646,9 +622,6 @@ const unixlib_entry_t __wine_unix_call_funcs[] =
     macdrv_dnd_have_format,
     macdrv_dnd_release,
     macdrv_dnd_retain,
-    macdrv_ime_clear,
-    macdrv_ime_process_text_input,
-    macdrv_ime_using_input_method,
     macdrv_init,
     macdrv_notify_icon,
     macdrv_quit_result,
@@ -676,28 +649,6 @@ static NTSTATUS wow64_dnd_get_data(void *arg)
     return macdrv_dnd_get_data(&params);
 }
 
-static NTSTATUS wow64_ime_process_text_input(void *arg)
-{
-    struct
-    {
-        UINT vkey;
-        UINT scan;
-        UINT repeat;
-        ULONG key_state;
-        ULONG himc;
-        ULONG done;
-    } *params32 = arg;
-    struct process_text_input_params params;
-
-    params.vkey = params32->vkey;
-    params.scan = params32->scan;
-    params.repeat = params32->repeat;
-    params.key_state = UlongToPtr(params32->key_state);
-    params.himc = UlongToPtr(params32->himc);
-    params.done = UlongToPtr(params32->done);
-    return macdrv_ime_process_text_input(&params);
-}
-
 static NTSTATUS wow64_init(void *arg)
 {
     struct
@@ -772,9 +723,6 @@ const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
     macdrv_dnd_have_format,
     macdrv_dnd_release,
     macdrv_dnd_retain,
-    macdrv_ime_clear,
-    wow64_ime_process_text_input,
-    macdrv_ime_using_input_method,
     wow64_init,
     wow64_notify_icon,
     macdrv_quit_result,
