@@ -45,6 +45,8 @@ struct wayland_win_data
     struct window_surface *window_surface;
     /* USER window rectangle relative to win32 parent window client area */
     RECT window_rect;
+    /* USER client rectangle relative to win32 parent window client area */
+    RECT client_rect;
 };
 
 static int wayland_win_data_cmp_rb(const void *key,
@@ -68,7 +70,8 @@ static struct rb_tree win_data_rb = { wayland_win_data_cmp_rb };
  * Create a data window structure for an existing window.
  */
 static struct wayland_win_data *wayland_win_data_create(HWND hwnd,
-                                                        const RECT *window_rect)
+                                                        const RECT *window_rect,
+                                                        const RECT *client_rect)
 {
     struct wayland_win_data *data;
     struct rb_entry *rb_entry;
@@ -83,6 +86,7 @@ static struct wayland_win_data *wayland_win_data_create(HWND hwnd,
 
     data->hwnd = hwnd;
     data->window_rect = *window_rect;
+    data->client_rect = *client_rect;
 
     pthread_mutex_lock(&win_data_mutex);
 
@@ -157,6 +161,7 @@ static void wayland_win_data_get_config(struct wayland_win_data *data,
     DWORD style;
 
     conf->rect = data->window_rect;
+    conf->client_rect = data->client_rect;
     style = NtUserGetWindowLongW(data->hwnd, GWL_STYLE);
 
     TRACE("window=%s style=%#lx\n", wine_dbgstr_rect(&conf->rect), (long)style);
@@ -175,6 +180,7 @@ static void wayland_win_data_get_config(struct wayland_win_data *data,
     }
 
     conf->state = window_state;
+    conf->scale = NtUserGetDpiForWindow(data->hwnd) / 96.0;
 }
 
 static void wayland_win_data_update_wayland_surface(struct wayland_win_data *data)
@@ -309,7 +315,8 @@ BOOL WAYLAND_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags,
           hwnd, wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
           wine_dbgstr_rect(visible_rect), insert_after, swp_flags);
 
-    if (!data && !(data = wayland_win_data_create(hwnd, window_rect))) return TRUE;
+    if (!data && !(data = wayland_win_data_create(hwnd, window_rect, client_rect)))
+        return TRUE;
 
     /* Release the dummy surface wine provides for toplevels. */
     if (*surface) window_surface_release(*surface);
@@ -360,6 +367,7 @@ void WAYLAND_WindowPosChanged(HWND hwnd, HWND insert_after, UINT swp_flags,
     if (!data) return;
 
     data->window_rect = *window_rect;
+    data->client_rect = *client_rect;
 
     if (surface) window_surface_add_ref(surface);
     if (data->window_surface) window_surface_release(data->window_surface);
@@ -384,7 +392,8 @@ static void wayland_resize_desktop(void)
 static void wayland_configure_window(HWND hwnd)
 {
     struct wayland_surface *surface;
-    INT width, height;
+    INT width, height, window_width, window_height;
+    INT window_surf_width, window_surf_height;
     UINT flags = 0;
     uint32_t state;
     DWORD style;
@@ -410,9 +419,18 @@ static void wayland_configure_window(HWND hwnd)
     surface->processing = surface->requested;
     memset(&surface->requested, 0, sizeof(surface->requested));
 
-    width = surface->processing.width;
-    height = surface->processing.height;
     state = surface->processing.state;
+    /* Ignore size hints if we don't have a state that requires strict
+     * size adherence, in order to avoid spurious resizes. */
+    if (state)
+    {
+        width = surface->processing.width;
+        height = surface->processing.height;
+    }
+    else
+    {
+        width = height = 0;
+    }
 
     if ((state & WAYLAND_SURFACE_CONFIG_STATE_RESIZING) && !surface->resizing)
     {
@@ -434,21 +452,28 @@ static void wayland_configure_window(HWND hwnd)
         flags |= SWP_FRAMECHANGED;
     }
 
+    wayland_surface_coords_from_window(surface,
+                                       surface->window.rect.right -
+                                           surface->window.rect.left,
+                                       surface->window.rect.bottom -
+                                           surface->window.rect.top,
+                                       &window_surf_width, &window_surf_height);
+
     /* If the window is already fullscreen and its size is compatible with what
      * the compositor is requesting, don't force a resize, since some applications
      * are very insistent on a particular fullscreen size (which may not match
      * the monitor size). */
     if ((surface->window.state & WAYLAND_SURFACE_CONFIG_STATE_FULLSCREEN) &&
         wayland_surface_config_is_compatible(&surface->processing,
-                                             surface->window.rect.right -
-                                                surface->window.rect.left,
-                                             surface->window.rect.bottom -
-                                                surface->window.rect.top,
+                                             window_surf_width, window_surf_height,
                                              surface->window.state))
     {
         flags |= SWP_NOSIZE;
     }
 
+    wayland_surface_coords_to_window(surface, width, height,
+                                     &window_width, &window_height);
+
     pthread_mutex_unlock(&surface->mutex);
 
     TRACE("processing=%dx%d,%#x\n", width, height, state);
@@ -457,7 +482,7 @@ static void wayland_configure_window(HWND hwnd)
     if (needs_exit_size_move) send_message(hwnd, WM_EXITSIZEMOVE, 0, 0);
 
     flags |= SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOOWNERZORDER | SWP_NOMOVE;
-    if (width == 0 || height == 0) flags |= SWP_NOSIZE;
+    if (window_width == 0 || window_height == 0) flags |= SWP_NOSIZE;
 
     style = NtUserGetWindowLongW(hwnd, GWL_STYLE);
     if (!(state & WAYLAND_SURFACE_CONFIG_STATE_MAXIMIZED) != !(style & WS_MAXIMIZE))
@@ -474,7 +499,7 @@ static void wayland_configure_window(HWND hwnd)
         flags |= SWP_NOSENDCHANGING;
     }
 
-    NtUserSetWindowPos(hwnd, 0, 0, 0, width, height, flags);
+    NtUserSetWindowPos(hwnd, 0, 0, 0, window_width, window_height, flags);
 }
 
 /**********************************************************************
