@@ -94,13 +94,11 @@ static void color_or_size_dsa_add (WrappedDsa* wdsa, const WCHAR* name,
 				   const WCHAR* fancyName)
 {
     ThemeColorOrSize item;
-    
-    item.name = HeapAlloc (GetProcessHeap(), 0, 
-	(lstrlenW (name) + 1) * sizeof(WCHAR));
+
+    item.name = malloc ((wcslen (name) + 1) * sizeof(WCHAR));
     lstrcpyW (item.name, name);
 
-    item.fancyName = HeapAlloc (GetProcessHeap(), 0, 
-	(lstrlenW (fancyName) + 1) * sizeof(WCHAR));
+    item.fancyName = malloc ((wcslen (fancyName) + 1) * sizeof(WCHAR));
     lstrcpyW (item.fancyName, fancyName);
 
     DSA_InsertItem (wdsa->dsa, wdsa->count, &item);
@@ -110,8 +108,8 @@ static void color_or_size_dsa_add (WrappedDsa* wdsa, const WCHAR* name,
 static int CALLBACK dsa_destroy_callback (LPVOID p, LPVOID pData)
 {
     ThemeColorOrSize* item = p;
-    HeapFree (GetProcessHeap(), 0, item->name);
-    HeapFree (GetProcessHeap(), 0, item->fancyName);
+    free (item->name);
+    free (item->fancyName);
     return 1;
 }
 
@@ -157,8 +155,8 @@ static int themeFilesCount = 0;
 static int CALLBACK theme_dsa_destroy_callback (LPVOID p, LPVOID pData)
 {
     ThemeFile* item = p;
-    HeapFree (GetProcessHeap(), 0, item->themeFileName);
-    HeapFree (GetProcessHeap(), 0, item->fancyName);
+    free (item->themeFileName);
+    free (item->fancyName);
     free_color_or_size_dsa (&item->colors);
     free_color_or_size_dsa (&item->sizes);
     return 1;
@@ -209,12 +207,10 @@ static BOOL CALLBACK myEnumThemeProc (LPVOID lpReserved,
     create_color_or_size_dsa (&newEntry.sizes);
     fill_theme_string_array (pszThemeFileName, &newEntry.sizes, EnumThemeSizes);
 
-    newEntry.themeFileName = HeapAlloc (GetProcessHeap(), 0, 
-	(lstrlenW (pszThemeFileName) + 1) * sizeof(WCHAR));
+    newEntry.themeFileName = malloc ((wcslen (pszThemeFileName) + 1) * sizeof(WCHAR));
     lstrcpyW (newEntry.themeFileName, pszThemeFileName);
-  
-    newEntry.fancyName = HeapAlloc (GetProcessHeap(), 0, 
-	(lstrlenW (pszThemeName) + 1) * sizeof(WCHAR));
+
+    newEntry.fancyName = malloc ((wcslen (pszThemeName) + 1) * sizeof(WCHAR));
     lstrcpyW (newEntry.fancyName, pszThemeName);
   
     /*list_add_tail (&themeFiles, &newEntry->entry);*/
@@ -422,11 +418,48 @@ static void enable_size_and_color_controls (HWND dialog, BOOL enable)
     EnableWindow (GetDlgItem (dialog, IDC_THEME_SIZECOMBO), enable);
     EnableWindow (GetDlgItem (dialog, IDC_THEME_SIZETEXT), enable);
 }
+
+static DWORD get_app_theme(void)
+{
+    DWORD ret = 0, len = sizeof(ret), type;
+    HKEY hkey;
+
+    if (RegOpenKeyExW( HKEY_CURRENT_USER, L"Software\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize", 0, KEY_QUERY_VALUE, &hkey ))
+        return 1;
+    if (RegQueryValueExW( hkey, L"AppsUseLightTheme", NULL, &type, (BYTE *)&ret, &len ) || type != REG_DWORD)
+        ret = 1;
+
+    RegCloseKey( hkey );
+    return ret;
+}
   
 static void init_dialog (HWND dialog)
 {
-    SendDlgItemMessageW(dialog, IDC_SYSPARAM_SIZE_UD, UDM_SETBUDDY,
-                        (WPARAM)GetDlgItem(dialog, IDC_SYSPARAM_SIZE), 0);
+    DWORD apps_use_light_theme;
+    WCHAR apps_theme_str[256];
+
+    static const struct
+    {
+        int id;
+        DWORD value;
+    }
+    app_themes[] =
+    {
+        { IDC_THEME_APPCOMBO_LIGHT, 1 },
+        { IDC_THEME_APPCOMBO_DARK,  0 },
+    };
+
+    SendDlgItemMessageW( dialog, IDC_THEME_APPCOMBO, CB_RESETCONTENT, 0, 0 );
+
+    LoadStringW( GetModuleHandleW(NULL), app_themes[0].id, apps_theme_str, ARRAY_SIZE(apps_theme_str) );
+    SendDlgItemMessageW( dialog, IDC_THEME_APPCOMBO, CB_ADDSTRING, 0, (LPARAM)apps_theme_str );
+    LoadStringW( GetModuleHandleW(NULL), app_themes[1].id, apps_theme_str, ARRAY_SIZE(apps_theme_str) );
+    SendDlgItemMessageW( dialog, IDC_THEME_APPCOMBO, CB_ADDSTRING, 0, (LPARAM)apps_theme_str );
+
+    apps_use_light_theme = get_app_theme();
+    SendDlgItemMessageW( dialog, IDC_THEME_APPCOMBO, CB_SETCURSEL, app_themes[apps_use_light_theme].value, 0 );
+
+    SendDlgItemMessageW( dialog, IDC_SYSPARAM_SIZE_UD, UDM_SETBUDDY, (WPARAM)GetDlgItem(dialog, IDC_SYSPARAM_SIZE), 0 );
 }
 
 static void update_dialog (HWND dialog)
@@ -454,8 +487,9 @@ static void update_dialog (HWND dialog)
 }
 
 static void on_theme_changed(HWND dialog) {
-    int index = SendMessageW (GetDlgItem (dialog, IDC_THEME_THEMECOMBO),
-        CB_GETCURSEL, 0, 0);
+    int index;
+
+    index = SendMessageW (GetDlgItem (dialog, IDC_THEME_THEMECOMBO), CB_GETCURSEL, 0, 0);
     if (!update_color_and_size (index, GetDlgItem (dialog, IDC_THEME_COLORCOMBO),
         GetDlgItem (dialog, IDC_THEME_SIZECOMBO)))
     {
@@ -467,6 +501,13 @@ static void on_theme_changed(HWND dialog) {
     {
         enable_size_and_color_controls (dialog, TRUE);
     }
+
+    index = SendMessageW (GetDlgItem (dialog, IDC_THEME_APPCOMBO), CB_GETCURSEL, 0, 0);
+    set_reg_key_dword(HKEY_CURRENT_USER, L"Software\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize",
+                      L"AppsUseLightTheme", !index);
+    set_reg_key_dword(HKEY_CURRENT_USER, L"Software\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize",
+                      L"SystemUsesLightTheme", !index);
+
     theme_dirty = TRUE;
 }
 
@@ -800,7 +841,7 @@ static void update_shell_folder_listview(HWND dialog) {
         item.iSubItem = 1;
         item.pszText = strdupU2W(asfiInfo[i].szLinkTarget);
         SendDlgItemMessageW(dialog, IDC_LIST_SFPATHS, LVM_SETITEMW, 0, (LPARAM)&item);
-        HeapFree(GetProcessHeap(), 0, item.pszText);
+        free(item.pszText);
     }
 
     /* Ensure that the previously selected item is selected again. */
@@ -823,7 +864,7 @@ static void on_shell_folder_selection_changed(HWND hDlg, LPNMLISTVIEW lpnm) {
             EnableWindow(GetDlgItem(hDlg, IDC_BROWSE_SFPATH), 1);
             link = strdupU2W(psfiSelected->szLinkTarget);
             set_textW(hDlg, IDC_EDIT_SFPATH, link);
-            HeapFree(GetProcessHeap(), 0, link);
+            free(link);
         } else {
             CheckDlgButton(hDlg, IDC_LINK_SFPATH, BST_UNCHECKED);
             EnableWindow(GetDlgItem(hDlg, IDC_EDIT_SFPATH), 0);
@@ -849,7 +890,7 @@ static void on_shell_folder_edit_changed(HWND hDlg) {
                                     MAKELPARAM(LVNI_SELECTED,0));
 
     if (!text || !psfiSelected || iSel < 0) {
-        HeapFree(GetProcessHeap(), 0, text);
+        free(text);
         return;
     }
 
@@ -862,7 +903,7 @@ static void on_shell_folder_edit_changed(HWND hDlg) {
     item.pszText = text;
     SendDlgItemMessageW(hDlg, IDC_LIST_SFPATHS, LVM_SETITEMW, 0, (LPARAM)&item);
 
-    HeapFree(GetProcessHeap(), 0, text);
+    free(text);
 
     SendMessageW(GetParent(hDlg), PSM_CHANGED, 0, 0);
 }
@@ -1094,7 +1135,7 @@ static void init_mime_types(HWND hDlg)
 
     CheckDlgButton(hDlg, IDC_ENABLE_FILE_ASSOCIATIONS, state);
 
-    HeapFree(GetProcessHeap(), 0, buf);
+    free(buf);
 }
 
 static void update_mime_types(HWND hDlg)
@@ -1152,6 +1193,7 @@ ThemeDlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                     if (updating_ui) break;
                     switch (LOWORD(wParam))
                     {
+                        case IDC_THEME_APPCOMBO: /* fall through */
                         case IDC_THEME_THEMECOMBO: on_theme_changed(hDlg); break;
                         case IDC_THEME_COLORCOMBO: /* fall through */
                         case IDC_THEME_SIZECOMBO: theme_dirty = TRUE; break;
@@ -1175,7 +1217,7 @@ ThemeDlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                             if (text)
                             {
                                 metrics[index].size = wcstol(text, NULL, 10);
-                                HeapFree(GetProcessHeap(), 0, text);
+                                free(text);
                             }
                             else
                             {
