@@ -19,16 +19,15 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
+#include <fcntl.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_SYS_MMAN_H
-# include <sys/mman.h>
-#endif
+#include <sys/mman.h>
 #include <unistd.h>
 
 #include "ntstatus.h"
@@ -204,14 +203,36 @@ static const struct fd_ops mapping_fd_ops =
     no_fd_get_file_info,          /* get_file_info */
     no_fd_get_volume_info,        /* get_volume_info */
     no_fd_ioctl,                  /* ioctl */
+    default_fd_cancel_async,      /* cancel_async */
     no_fd_queue_async,            /* queue_async */
     default_fd_reselect_async     /* reselect_async */
 };
 
+/* free address ranges for PE image mappings */
+struct addr_range
+{
+    unsigned int count;
+    unsigned int size;
+    struct
+    {
+        client_ptr_t base;
+        mem_size_t size;
+    } *free;
+};
+
 static size_t page_mask;
+static const mem_size_t granularity_mask = 0xffff;
+static struct addr_range ranges32;
+static struct addr_range ranges64;
 
 #define ROUND_SIZE(size)  (((size) + page_mask) & ~page_mask)
 
+void init_memory(void)
+{
+    page_mask = sysconf( _SC_PAGESIZE ) - 1;
+    free_map_addr( 0x60000000, 0x1c000000 );
+    free_map_addr( 0x600000000000, 0x100000000000 );
+}
 
 static void ranges_dump( struct object *obj, int verbose )
 {
@@ -241,7 +262,7 @@ static void shared_map_destroy( struct object *obj )
 }
 
 /* extend a file beyond the current end of file */
-static int grow_file( int unix_fd, file_pos_t new_size )
+int grow_file( int unix_fd, file_pos_t new_size )
 {
     static const char zero;
     off_t size = new_size;
@@ -262,14 +283,29 @@ static int grow_file( int unix_fd, file_pos_t new_size )
     return 0;
 }
 
+/* simplified version of mkstemps() */
+static int make_temp_file( char name[16] )
+{
+    static unsigned int value;
+    int i, fd = -1;
+
+    value += (current_time >> 16) + current_time;
+    for (i = 0; i < 0x8000 && fd < 0; i++, value += 7777)
+    {
+        sprintf( name, "tmpmap-%08x", value );
+        fd = open( name, O_RDWR | O_CREAT | O_EXCL, 0600 );
+    }
+    return fd;
+}
+
 /* check if the current directory allows exec mappings */
 static int check_current_dir_for_exec(void)
 {
     int fd;
-    char tmpfn[] = "anonmap.XXXXXX";
+    char tmpfn[16];
     void *ret = MAP_FAILED;
 
-    fd = mkstemps( tmpfn, 0 );
+    fd = make_temp_file( tmpfn );
     if (fd == -1) return 0;
     if (grow_file( fd, 1 ))
     {
@@ -285,7 +321,7 @@ static int check_current_dir_for_exec(void)
 static int create_temp_file( file_pos_t size )
 {
     static int temp_dir_fd = -1;
-    char tmpfn[] = "anonmap.XXXXXX";
+    char tmpfn[16];
     int fd;
 
     if (temp_dir_fd == -1)
@@ -303,7 +339,7 @@ static int create_temp_file( file_pos_t size )
     }
     else if (temp_dir_fd != server_dir_fd) fchdir( temp_dir_fd );
 
-    fd = mkstemps( tmpfn, 0 );
+    fd = make_temp_file( tmpfn );
     if (fd != -1)
     {
         if (!grow_file( fd, size ))
@@ -343,12 +379,46 @@ static struct memory_view *find_mapped_addr( struct process *process, client_ptr
     return NULL;
 }
 
+/* check if an address range is valid for creating a view */
+static int is_valid_view_addr( struct process *process, client_ptr_t addr, mem_size_t size )
+{
+    struct memory_view *view;
+
+    if (!size) return 0;
+    if (addr & page_mask) return 0;
+    if (addr + size < addr) return 0;  /* overflow */
+
+    /* check for overlapping view */
+    LIST_FOR_EACH_ENTRY( view, &process->views, struct memory_view, entry )
+    {
+        if (view->base + view->size <= addr) continue;
+        if (view->base >= addr + size) continue;
+        return 0;
+    }
+    return 1;
+}
+
 /* get the main exe memory view */
 struct memory_view *get_exe_view( struct process *process )
 {
     return LIST_ENTRY( list_head( &process->views ), struct memory_view, entry );
 }
 
+static void set_process_machine( struct process *process, struct memory_view *view )
+{
+    if (view->image.image_flags & IMAGE_FLAGS_ComPlusNativeReady)
+        process->machine = native_machine;
+    else
+        process->machine = view->image.machine;
+}
+
+static int generate_dll_event( struct thread *thread, int code, struct memory_view *view )
+{
+    if (!(view->flags & SEC_IMAGE)) return 0;
+    generate_debug_event( thread, code, view );
+    return 1;
+}
+
 /* add a view to the process list */
 static void add_process_view( struct thread *thread, struct memory_view *view )
 {
@@ -358,14 +428,20 @@ static void add_process_view( struct thread *thread, struct memory_view *view )
     if (view->flags & SEC_IMAGE)
     {
         if (is_process_init_done( process ))
-            generate_debug_event( thread, DbgLoadDllStateChange, view );
+        {
+            generate_dll_event( thread, DbgLoadDllStateChange, view );
+        }
         else if (!(view->image.image_charact & IMAGE_FILE_DLL))
         {
             /* main exe */
-            process->machine = view->image.machine;
+            set_process_machine( process, view );
             list_add_head( &process->views, &view->entry );
+
+            free( process->image );
+            process->image = NULL;
             if (get_view_nt_name( view, &name ) && (process->image = memdup( name.str, name.len )))
                 process->imagelen = name.len;
+            process->image_info = view->image;
             return;
         }
     }
@@ -640,9 +716,9 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
         } opt;
     } nt;
     off_t pos;
-    int size, opt_size;
-    size_t mz_size, clr_va, clr_size;
-    unsigned int i, cpu_mask = get_supported_cpu_mask();
+    int size, has_relocs;
+    size_t mz_size, clr_va = 0, clr_size = 0;
+    unsigned int i;
 
     /* load the headers */
 
@@ -655,9 +731,6 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
 
     size = pread( unix_fd, &nt, sizeof(nt), pos );
     if (size < sizeof(nt.Signature) + sizeof(nt.FileHeader)) return STATUS_INVALID_IMAGE_PROTECT;
-    /* zero out Optional header in the case it's not present or partial */
-    opt_size = max( nt.FileHeader.SizeOfOptionalHeader, offsetof( IMAGE_OPTIONAL_HEADER32, CheckSum ));
-    size = min( size, sizeof(nt.Signature) + sizeof(nt.FileHeader) + opt_size );
     if (size < sizeof(nt)) memset( (char *)&nt + size, 0, sizeof(nt) - size );
     if (nt.Signature != IMAGE_NT_SIGNATURE)
     {
@@ -671,22 +744,25 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
     switch (nt.opt.hdr32.Magic)
     {
     case IMAGE_NT_OPTIONAL_HDR32_MAGIC:
-        switch (nt.FileHeader.Machine)
-        {
-        case IMAGE_FILE_MACHINE_I386:
-            if (cpu_mask & (CPU_FLAG(CPU_x86) | CPU_FLAG(CPU_x86_64))) break;
-            return STATUS_INVALID_IMAGE_FORMAT;
-        case IMAGE_FILE_MACHINE_ARMNT:
-            if (cpu_mask & (CPU_FLAG(CPU_ARM) | CPU_FLAG(CPU_ARM64))) break;
-            return STATUS_INVALID_IMAGE_FORMAT;
-        default:
-            return STATUS_INVALID_IMAGE_FORMAT;
-        }
-        clr_va = nt.opt.hdr32.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].VirtualAddress;
-        clr_size = nt.opt.hdr32.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].Size;
+        if (!is_machine_32bit( nt.FileHeader.Machine )) return STATUS_INVALID_IMAGE_FORMAT;
+        if (!is_machine_supported( nt.FileHeader.Machine )) return STATUS_INVALID_IMAGE_FORMAT;
 
+        if (nt.FileHeader.Machine != IMAGE_FILE_MACHINE_I386)  /* non-x86 platforms are more strict */
+        {
+            if (nt.opt.hdr32.SectionAlignment & page_mask)
+                return STATUS_INVALID_IMAGE_FORMAT;
+            if (!(nt.opt.hdr32.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_NX_COMPAT))
+                return STATUS_INVALID_IMAGE_FORMAT;
+            if (!(nt.opt.hdr32.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE))
+                return STATUS_INVALID_IMAGE_FORMAT;
+        }
+        if (nt.opt.hdr32.NumberOfRvaAndSizes > IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR)
+        {
+            clr_va = nt.opt.hdr32.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].VirtualAddress;
+            clr_size = nt.opt.hdr32.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].Size;
+        }
         mapping->image.base            = nt.opt.hdr32.ImageBase;
-        mapping->image.entry_point     = nt.opt.hdr32.ImageBase + nt.opt.hdr32.AddressOfEntryPoint;
+        mapping->image.entry_point     = nt.opt.hdr32.AddressOfEntryPoint;
         mapping->image.map_size        = ROUND_SIZE( nt.opt.hdr32.SizeOfImage );
         mapping->image.stack_size      = nt.opt.hdr32.SizeOfStackReserve;
         mapping->image.stack_commit    = nt.opt.hdr32.SizeOfStackCommit;
@@ -702,31 +778,39 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
         mapping->image.header_size     = nt.opt.hdr32.SizeOfHeaders;
         mapping->image.checksum        = nt.opt.hdr32.CheckSum;
         mapping->image.image_flags     = 0;
+
+        has_relocs = (nt.opt.hdr32.NumberOfRvaAndSizes > IMAGE_DIRECTORY_ENTRY_BASERELOC &&
+                      nt.opt.hdr32.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress &&
+                      nt.opt.hdr32.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size &&
+                      !(nt.FileHeader.Characteristics & IMAGE_FILE_RELOCS_STRIPPED));
         if (nt.opt.hdr32.SectionAlignment & page_mask)
             mapping->image.image_flags |= IMAGE_FLAGS_ImageMappedFlat;
-        if ((nt.opt.hdr32.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE) &&
-            mapping->image.contains_code && !(clr_va && clr_size))
+        else if ((nt.opt.hdr32.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE) &&
+                 (has_relocs || mapping->image.contains_code) && !(clr_va && clr_size))
             mapping->image.image_flags |= IMAGE_FLAGS_ImageDynamicallyRelocated;
         break;
 
     case IMAGE_NT_OPTIONAL_HDR64_MAGIC:
-        if (!(cpu_mask & CPU_64BIT_MASK)) return STATUS_INVALID_IMAGE_WIN_64;
-        switch (nt.FileHeader.Machine)
-        {
-        case IMAGE_FILE_MACHINE_AMD64:
-            if (cpu_mask & (CPU_FLAG(CPU_x86) | CPU_FLAG(CPU_x86_64))) break;
-            return STATUS_INVALID_IMAGE_FORMAT;
-        case IMAGE_FILE_MACHINE_ARM64:
-            if (cpu_mask & (CPU_FLAG(CPU_ARM) | CPU_FLAG(CPU_ARM64))) break;
-            return STATUS_INVALID_IMAGE_FORMAT;
-        default:
-            return STATUS_INVALID_IMAGE_FORMAT;
-        }
-        clr_va = nt.opt.hdr64.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].VirtualAddress;
-        clr_size = nt.opt.hdr64.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].Size;
+        if (!is_machine_64bit( native_machine )) return STATUS_INVALID_IMAGE_WIN_64;
+        if (!is_machine_64bit( nt.FileHeader.Machine )) return STATUS_INVALID_IMAGE_FORMAT;
+        if (!is_machine_supported( nt.FileHeader.Machine )) return STATUS_INVALID_IMAGE_FORMAT;
 
+        if (nt.FileHeader.Machine != IMAGE_FILE_MACHINE_AMD64)  /* non-x86 platforms are more strict */
+        {
+            if (nt.opt.hdr64.SectionAlignment & page_mask)
+                return STATUS_INVALID_IMAGE_FORMAT;
+            if (!(nt.opt.hdr64.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_NX_COMPAT))
+                return STATUS_INVALID_IMAGE_FORMAT;
+            if (!(nt.opt.hdr64.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE))
+                return STATUS_INVALID_IMAGE_FORMAT;
+        }
+        if (nt.opt.hdr64.NumberOfRvaAndSizes > IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR)
+        {
+            clr_va = nt.opt.hdr64.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].VirtualAddress;
+            clr_size = nt.opt.hdr64.DataDirectory[IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR].Size;
+        }
         mapping->image.base            = nt.opt.hdr64.ImageBase;
-        mapping->image.entry_point     = nt.opt.hdr64.ImageBase + nt.opt.hdr64.AddressOfEntryPoint;
+        mapping->image.entry_point     = nt.opt.hdr64.AddressOfEntryPoint;
         mapping->image.map_size        = ROUND_SIZE( nt.opt.hdr64.SizeOfImage );
         mapping->image.stack_size      = nt.opt.hdr64.SizeOfStackReserve;
         mapping->image.stack_commit    = nt.opt.hdr64.SizeOfStackCommit;
@@ -742,10 +826,15 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
         mapping->image.header_size     = nt.opt.hdr64.SizeOfHeaders;
         mapping->image.checksum        = nt.opt.hdr64.CheckSum;
         mapping->image.image_flags     = 0;
+
+        has_relocs = (nt.opt.hdr64.NumberOfRvaAndSizes > IMAGE_DIRECTORY_ENTRY_BASERELOC &&
+                      nt.opt.hdr64.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress &&
+                      nt.opt.hdr64.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size &&
+                      !(nt.FileHeader.Characteristics & IMAGE_FILE_RELOCS_STRIPPED));
         if (nt.opt.hdr64.SectionAlignment & page_mask)
             mapping->image.image_flags |= IMAGE_FLAGS_ImageMappedFlat;
-        if ((nt.opt.hdr64.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE) &&
-            mapping->image.contains_code && !(clr_va && clr_size))
+        else if ((nt.opt.hdr64.DllCharacteristics & IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE) &&
+                 (has_relocs || mapping->image.contains_code) && !(clr_va && clr_size))
             mapping->image.image_flags |= IMAGE_FLAGS_ImageDynamicallyRelocated;
         break;
 
@@ -753,6 +842,7 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
         return STATUS_INVALID_IMAGE_FORMAT;
     }
 
+    mapping->image.map_addr      = get_fd_map_address( mapping->fd );
     mapping->image.image_charact = nt.FileHeader.Characteristics;
     mapping->image.machine       = nt.FileHeader.Machine;
     mapping->image.dbg_offset    = nt.FileHeader.PointerToSymbolTable;
@@ -760,10 +850,10 @@ static unsigned int get_image_params( struct mapping *mapping, file_pos_t file_s
     mapping->image.zerobits      = 0; /* FIXME */
     mapping->image.file_size     = file_size;
     mapping->image.loader_flags  = clr_va && clr_size;
-    if (mz_size == sizeof(mz) && !memcmp( mz.buffer, builtin_signature, sizeof(builtin_signature) ))
-        mapping->image.image_flags |= IMAGE_FLAGS_WineBuiltin;
-    else if (mz_size == sizeof(mz) && !memcmp( mz.buffer, fakedll_signature, sizeof(fakedll_signature) ))
-        mapping->image.image_flags |= IMAGE_FLAGS_WineFakeDll;
+    mapping->image.wine_builtin  = (mz_size == sizeof(mz) &&
+                                    !memcmp( mz.buffer, builtin_signature, sizeof(builtin_signature) ));
+    mapping->image.wine_fakedll  = (mz_size == sizeof(mz) &&
+                                    !memcmp( mz.buffer, fakedll_signature, sizeof(fakedll_signature) ));
 
     /* load the section headers */
 
@@ -846,8 +936,6 @@ static struct mapping *create_mapping( struct object *root, const struct unicode
     int unix_fd;
     struct stat st;
 
-    if (!page_mask) page_mask = sysconf( _SC_PAGESIZE ) - 1;
-
     if (!(mapping = create_named_object( root, &mapping_ops, name, attr, sd )))
         return NULL;
     if (get_error() == STATUS_OBJECT_NAME_EXISTS)
@@ -1017,9 +1105,7 @@ void generate_startup_debug_events( struct process *process )
     while (ptr && (ptr = list_next( &process->views, ptr )))
     {
         view = LIST_ENTRY( ptr, struct memory_view, entry );
-        if (!(view->flags & SEC_IMAGE)) continue;
-        generate_debug_event( first_thread, DbgLoadDllStateChange, view );
-        break;
+        if (generate_dll_event( first_thread, DbgLoadDllStateChange, view )) break;
     }
 
     /* generate creation events */
@@ -1033,8 +1119,7 @@ void generate_startup_debug_events( struct process *process )
     while (ptr && (ptr = list_next( &process->views, ptr )))
     {
         view = LIST_ENTRY( ptr, struct memory_view, entry );
-        if (!(view->flags & SEC_IMAGE)) continue;
-        generate_debug_event( first_thread, DbgLoadDllStateChange, view );
+        generate_dll_event( first_thread, DbgLoadDllStateChange, view );
     }
 }
 
@@ -1067,9 +1152,70 @@ static enum server_fd_type mapping_get_fd_type( struct fd *fd )
     return FD_TYPE_FILE;
 }
 
+/* assign a mapping address to a PE image mapping */
+static client_ptr_t assign_map_address( struct mapping *mapping )
+{
+    unsigned int i;
+    client_ptr_t ret;
+    struct addr_range *range = (mapping->image.base >> 32) ? &ranges64 : &ranges32;
+    mem_size_t size = (mapping->size + granularity_mask) & ~granularity_mask;
+
+    if (!(mapping->image.image_charact & IMAGE_FILE_DLL)) return 0;
+
+    if ((ret = get_fd_map_address( mapping->fd ))) return ret;
+
+    for (i = 0; i < range->count; i++)
+    {
+        if (range->free[i].size < size) continue;
+        range->free[i].size -= size;
+        ret = range->free[i].base + range->free[i].size;
+        set_fd_map_address( mapping->fd, ret, size );
+        return ret;
+    }
+    return 0;
+}
+
+/* free a PE mapping address range when the last mapping is closed */
+void free_map_addr( client_ptr_t base, mem_size_t size )
+{
+    unsigned int i;
+    client_ptr_t end = base + size;
+    struct addr_range *range = (base >> 32) ? &ranges64 : &ranges32;
+
+    for (i = 0; i < range->count; i++)
+    {
+        if (range->free[i].base > end) continue;
+        if (range->free[i].base + range->free[i].size < base) break;
+        if (range->free[i].base == end)
+        {
+            if (i + 1 < range->count && range->free[i + 1].base + range->free[i + 1].size == base)
+            {
+                size += range->free[i].size;
+                range->count--;
+                memmove( &range->free[i], &range->free[i + 1], (range->count - i) * sizeof(*range->free) );
+            }
+            else range->free[i].base = base;
+        }
+        range->free[i].size += size;
+        return;
+    }
+
+    if (range->count == range->size)
+    {
+        unsigned int new_size = max( 256, range->size * 2 );
+        void *new_free = realloc( range->free, new_size * sizeof(*range->free) );
+        if (!new_free) return;
+        range->size = new_size;
+        range->free = new_free;
+    }
+    memmove( &range->free[i + 1], &range->free[i], (range->count - i) * sizeof(*range->free) );
+    range->free[i].base = base;
+    range->free[i].size = size;
+    range->count++;
+}
+
 int get_page_size(void)
 {
-    if (!page_mask) page_mask = sysconf( _SC_PAGESIZE ) - 1;
     return page_mask + 1;
 }
 
@@ -1163,92 +1309,168 @@ DECL_HANDLER(get_mapping_info)
     release_object( mapping );
 }
 
+/* get the address to use to map an image mapping */
+DECL_HANDLER(get_image_map_address)
+{
+    struct mapping *mapping;
+
+    if (!(mapping = get_mapping_obj( current->process, req->handle, SECTION_MAP_READ ))) return;
+
+    if ((mapping->flags & SEC_IMAGE) &&
+        (mapping->image.image_flags & IMAGE_FLAGS_ImageDynamicallyRelocated))
+    {
+        if (!mapping->image.map_addr) mapping->image.map_addr = assign_map_address( mapping );
+        reply->addr = mapping->image.map_addr;
+    }
+    else set_error( STATUS_INVALID_PARAMETER );
+
+    release_object( mapping );
+}
+
 /* add a memory view in the current process */
 DECL_HANDLER(map_view)
 {
-    struct mapping *mapping = NULL;
+    struct mapping *mapping;
     struct memory_view *view;
-    data_size_t namelen = 0;
 
-    if (!req->size || (req->base & page_mask) || req->base + req->size < req->base)  /* overflow */
+    if (!is_valid_view_addr( current->process, req->base, req->size ))
     {
         set_error( STATUS_INVALID_PARAMETER );
         return;
     }
 
-    /* make sure we don't already have an overlapping view */
-    LIST_FOR_EACH_ENTRY( view, &current->process->views, struct memory_view, entry )
-    {
-        if (view->base + view->size <= req->base) continue;
-        if (view->base >= req->base + req->size) continue;
-        set_error( STATUS_INVALID_PARAMETER );
-        return;
-    }
-
-    if (!req->mapping)  /* image mapping for a .so dll */
-    {
-        if (get_req_data_size() > sizeof(view->image)) namelen = get_req_data_size() - sizeof(view->image);
-        if (!(view = mem_alloc( offsetof( struct memory_view, name[namelen] )))) return;
-        memset( view, 0, sizeof(*view) );
-        view->base    = req->base;
-        view->size    = req->size;
-        view->start   = req->start;
-        view->flags   = SEC_IMAGE;
-        view->namelen = namelen;
-        memcpy( &view->image, get_req_data(), min( sizeof(view->image), get_req_data_size() ));
-        memcpy( view->name, (pe_image_info_t *)get_req_data() + 1, namelen );
-        add_process_view( current, view );
-        return;
-    }
-
     if (!(mapping = get_mapping_obj( current->process, req->mapping, req->access ))) return;
 
-    if (mapping->flags & SEC_IMAGE)
-    {
-        if (req->start || req->size > mapping->image.map_size)
-        {
-            set_error( STATUS_INVALID_PARAMETER );
-            goto done;
-        }
-    }
-    else if (req->start >= mapping->size ||
-             req->start + req->size < req->start ||
-             req->start + req->size > ((mapping->size + page_mask) & ~(mem_size_t)page_mask))
+    if ((mapping->flags & SEC_IMAGE) ||
+        req->start >= mapping->size ||
+        req->start + req->size < req->start ||
+        req->start + req->size > ((mapping->size + page_mask) & ~(mem_size_t)page_mask))
     {
         set_error( STATUS_INVALID_PARAMETER );
         goto done;
     }
 
-    if ((view = mem_alloc( offsetof( struct memory_view, name[namelen] ))))
+    if ((view = mem_alloc( sizeof(*view) )))
     {
         view->base      = req->base;
         view->size      = req->size;
         view->start     = req->start;
         view->flags     = mapping->flags;
-        view->namelen   = namelen;
+        view->namelen   = 0;
         view->fd        = !is_fd_removable( mapping->fd ) ? (struct fd *)grab_object( mapping->fd ) : NULL;
         view->committed = mapping->committed ? (struct ranges *)grab_object( mapping->committed ) : NULL;
-        view->shared    = mapping->shared ? (struct shared_map *)grab_object( mapping->shared ) : NULL;
-        if (view->flags & SEC_IMAGE) view->image = mapping->image;
+        view->shared    = NULL;
         add_process_view( current, view );
-        if (view->flags & SEC_IMAGE && view->base != mapping->image.base)
-            set_error( STATUS_IMAGE_NOT_AT_BASE );
     }
 
 done:
     release_object( mapping );
 }
 
+/* add a memory view for an image mapping in the current process */
+DECL_HANDLER(map_image_view)
+{
+    struct mapping *mapping;
+    struct memory_view *view;
+
+    if (!is_valid_view_addr( current->process, req->base, req->size ))
+    {
+        set_error( STATUS_INVALID_PARAMETER );
+        return;
+    }
+
+    if (!(mapping = get_mapping_obj( current->process, req->mapping, SECTION_MAP_READ ))) return;
+
+    if (!(mapping->flags & SEC_IMAGE) || req->size > mapping->image.map_size)
+    {
+        set_error( STATUS_INVALID_PARAMETER );
+        goto done;
+    }
+
+    if ((view = mem_alloc( sizeof(*view) )))
+    {
+        view->base      = req->base;
+        view->size      = req->size;
+        view->flags     = mapping->flags;
+        view->start     = 0;
+        view->namelen   = 0;
+        view->fd        = !is_fd_removable( mapping->fd ) ? (struct fd *)grab_object( mapping->fd ) : NULL;
+        view->committed = NULL;
+        view->shared    = mapping->shared ? (struct shared_map *)grab_object( mapping->shared ) : NULL;
+        view->image     = mapping->image;
+        view->image.machine     = req->machine;
+        view->image.entry_point = req->entry;
+        add_process_view( current, view );
+
+        if (view->base != (mapping->image.map_addr ? mapping->image.map_addr : mapping->image.base))
+            set_error( STATUS_IMAGE_NOT_AT_BASE );
+        if (view->image.machine != current->process->machine)
+        {
+            /* on 32-bit, the native 64-bit machine is allowed */
+            if (is_machine_64bit( current->process->machine ) || view->image.machine != native_machine)
+                set_error( STATUS_IMAGE_MACHINE_TYPE_MISMATCH );
+        }
+    }
+
+done:
+    release_object( mapping );
+}
+
+/* add a memory view for a builtin dll in the current process */
+DECL_HANDLER(map_builtin_view)
+{
+    struct memory_view *view;
+    const pe_image_info_t *image = get_req_data();
+    data_size_t namelen = get_req_data_size() - sizeof(*image);
+
+    if (get_req_data_size() < sizeof(*image) ||
+        (namelen & (sizeof(WCHAR) - 1)) ||
+        !is_valid_view_addr( current->process, image->base, image->map_size ))
+    {
+        set_error( STATUS_INVALID_PARAMETER );
+        return;
+    }
+
+    if ((view = mem_alloc( sizeof(struct memory_view) + namelen )))
+    {
+        memset( view, 0, sizeof(*view) );
+        view->base    = image->base;
+        view->size    = image->map_size;
+        view->flags   = SEC_IMAGE;
+        view->image   = *image;
+        view->namelen = namelen;
+        memcpy( view->name, image + 1, namelen );
+        add_process_view( current, view );
+    }
+}
+
 /* unmap a memory view from the current process */
 DECL_HANDLER(unmap_view)
 {
     struct memory_view *view = find_mapped_view( current->process, req->base );
 
     if (!view) return;
-    if (view->flags & SEC_IMAGE) generate_debug_event( current, DbgUnloadDllStateChange, view );
+    generate_dll_event( current, DbgUnloadDllStateChange, view );
     free_memory_view( view );
 }
 
+/* get information about a mapped image view */
+DECL_HANDLER(get_image_view_info)
+{
+    struct process *process;
+    struct memory_view *view;
+
+    if (!(process = get_process_from_handle( req->process, PROCESS_QUERY_INFORMATION ))) return;
+
+    if ((view = find_mapped_addr( process, req->addr )) && (view->flags & SEC_IMAGE))
+    {
+        reply->base = view->base;
+        reply->size = view->size;
+    }
+
+    release_object( process );
+}
+
 /* get a range of committed pages in a file mapping */
 DECL_HANDLER(get_mapping_committed_range)
 {
@@ -1272,9 +1494,7 @@ DECL_HANDLER(is_same_mapping)
     struct memory_view *view2 = find_mapped_view( current->process, req->base2 );
 
     if (!view1 || !view2) return;
-    if (!view1->fd || !view2->fd ||
-        !(view1->flags & SEC_IMAGE) || !(view2->flags & SEC_IMAGE) ||
-        !is_same_file_fd( view1->fd, view2->fd ))
+    if (!view1->fd || !view2->fd || !(view1->flags & SEC_IMAGE) || !is_same_file_fd( view1->fd, view2->fd ))
         set_error( STATUS_NOT_SAME_DEVICE );
 }
 
